% MORLAB - Model Order Reduction LABoratory.
% Version 4.0 06-December-2018
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%
% Model Reduction Methods for Standard Systems.
%   ml_ct_ss_brbt   - bounded-real balanced truncation (BRBT)
%   ml_ct_ss_bst    - balanced stochastic truncation (BST)
%   ml_ct_ss_bt     - balanced truncation (BT)
%   ml_ct_ss_flbt   - frequency-limited balanced truncation (FLBT)
%   ml_ct_ss_hinfbt - H-infinity balanced truncation (HinfBT)
%   ml_ct_ss_hna    - Hankel-norm approximation (HNA)
%   ml_ct_ss_lqgbt  - linear-quadratic Gaussian balanced truncation (LQGBT)
%   ml_ct_ss_mt     - modal truncation (MT)
%   ml_ct_ss_prbt   - positive-real balanced truncation (PRBT)
%   ml_ct_ss_tlbt   - time-limited balanced trucantion (TLBT)
%
% Model Reduction Methods for Descriptor Systems.
%   ml_ct_dss_brbt   - bounded-real balanced truncation (BRBT)
%   ml_ct_dss_bst    - balanced stochastic truncation (BST)
%   ml_ct_dss_bt     - balanced truncation (BT)
%   ml_ct_dss_flbt   - frequency-limited balanaced truncation (FLBT)
%   ml_ct_dss_hinfbt - H-infinity balanced truncation (HinfBT)
%   ml_ct_dss_hna    - Hankel-norm approximation (HNA)
%   ml_ct_dss_lqgbt  - linear-quadratic Gaussian balanced truncation (LQGBT)
%   ml_ct_dss_mt     - modal truncation (MT)
%   ml_ct_dss_prbt   - positive-real balanced truncation (PRBT)
%   ml_ct_dss_tlbt   - time-limited balanced truncation (TLBT)
%
% Model Reduction Methods for Second-Order Systems.
%   ml_ct_soss_bt   - balanced truncation (BT)
%   ml_ct_soss_flbt - frequency-limited balanced truncation (FLBT)
%   ml_ct_soss_tlbt - time-limited balanced truncation (TLBT)
%
% Demos: MOR for Standard Systems.
%   morlab_demo_ct_ss_brbt   - demo bounded-real balanced truncation
%   morlab_demo_ct_ss_bst    - demo balanced stochastic truncation
%   morlab_demo_ct_ss_bt     - demo balanced truncation
%   morlab_demo_ct_ss_flbt   - demo frequency-limited balanced truncation
%   morlab_demo_ct_ss_hinfbt - demo H-infinity balanced truncation
%   morlab_demo_ct_ss_hna    - demo Hankel-norm approximation
%   morlab_demo_ct_ss_lqgbt  - demo LQG balanced truncation
%   morlab_demo_ct_ss_mt     - demo modal truncation
%   morlab_demo_ct_ss_prbt   - demo positive-real balanced truncation
%   morlab_demo_ct_ss_tlbt   - demo time-limited balanced truncation
%
% Demos: MOR for Descriptor Systems.
%   morlab_demo_ct_dss_brbt   - demo bounded-real balanced truncation
%   morlab_demo_ct_dss_bst    - demo balanced stochastic truncation
%   morlab_demo_ct_dss_bt     - demo balanced truncation
%   morlab_demo_ct_dss_flbt   - demo frequency-limited balanced truncation
%   morlab_demo_ct_dss_hinfbt - demo H-infinity balanced truncation
%   morlab_demo_ct_dss_hna    - demo Hankel-norm approximation
%   morlab_demo_ct_dss_lqgbt  - demo LQG balanced truncation
%   morlab_demo_ct_dss_mt     - demo modal truncation
%   morlab_demo_ct_dss_prbt   - demo positive-real balanced truncation
%   morlab_demo_ct_dss_tlbt   - demo time-limited balanced truncation
%
% Demos: MOR for Second-Order Systems.
%   morlab_demo_ct_soss_bt   - demo balanced truncation
%   morlab_demo_ct_soss_flbt - demo frequency-limited balanced truncation
%   morlab_demo_ct_soss_tlbt - demo time-limited balanced truncation
%
% Matrix Equation Solvers.
%   ml_abe_sgn          - algebraic Bernoulli equation solver
%   ml_care_nwt_fac     - continuous-time Riccati equation solver (fac.)
%   ml_caredl_sgn       - dual continuous-time Riccati equation solver
%   ml_gdlyap_smith_fac - nilpotent discrete-time Lyapunov equation solver (fac.)
%   ml_icare_ric_fac    - continuous-time Hinf Riccati equation solver (fac.)
%   ml_lyap_sgn         - continuous-time Lyapunov equation solver
%   ml_lyap_sgn_fac     - continuous-time Lyapunov equation solver (fac.)
%   ml_lyap_sgn_ldl     - continuous-time Lyapunov equation solver (LDL)
%   ml_lyapdl_sgn_fac   - dual continuous-time Lyapunov equation solver (fac.)
%   ml_lyapdl_sgn_ldl   - dual continuous-time Lyapunov equation solver (LDL)
%   ml_pcare_nwt_fac    - positive continuous-time Riccati equation solver (fac.)
%   ml_sylv_sgn         - continuous-time Sylvester equation solver
%
% Subroutines.
%   ml_compress_fac       - SVD-based row/column compression
%   ml_compress_ldl       - LDL^T row/column compression.
%   ml_ct_dss_adtf        - Add. decomposition of transfer function (descriptor)
%   ml_ct_dss_bfsr        - balancing-free square root method (descriptor)
%   ml_ct_dss_lti2struct  - input to LTI struct conversion (descriptor)
%   ml_ct_dss_partstab    - partial stabilization (descriptor)
%   ml_ct_dss_sr          - square root method (descriptor)
%   ml_ct_soss_bfsr       - balancing-free square root method (second-order)
%   ml_ct_soss_lti2struct - input to LTI struct conversion (second-order)
%   ml_ct_soss_sr         - square root method (second-order)
%   ml_ct_ss_adtf         - Add. decomposition of transfer function (standard)
%   ml_ct_ss_bfsr         - balancing-free square root method (standard)
%   ml_ct_ss_lti2struct   - input to LTI struct conversion (standard)
%   ml_ct_ss_partstab     - partial stabilization (standard)
%   ml_ct_ss_sr           - square root method (standard)
%   ml_disk               - inverse-free iteration for matrix disk function
%   ml_getqz              - subspace extraction method
%   ml_morlabopts         - option struct generator
%   ml_order              - order computation for model reduction methods
%   ml_sigmaplot          - sigma plot of error transfer function
%   ml_signm              - matrix sign function iteration
%
% Checks.
%   ml_assert_boolean      - checks given value is boolean
%   ml_assert_descsys      - checks given structure is descriptor system
%   ml_assert_infdescsys   - checks given structure has inf matrices
%   ml_assert_integer      - checks given value is integer
%   ml_assert_nonnegint    - checks given value is nonnegative integer
%   ml_assert_nonnegintinf - checks given value is nonneg. integer or Inf
%   ml_assert_nonnegscalar - checks given value is nonnegative
%   ml_assert_nonnegvector - checks given vector is nonnegative
%   ml_assert_posinteger   - checks given value is positive integer
%   ml_assert_posscalar    - checks given value is positive
%   ml_assert_scalar       - checks given value is a scalar
%   ml_assert_sosys        - checks given structure is second-order system
%   ml_assert_stdsys       - checks given structure is standard system
%   ml_field_set_to_value  - checks given structure for non-empty field

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%
