%% BST for descriptor systems
%
% This demo script contains the application of the generalized balanced
% stochastic truncation method (ml_ct_dss_bst) on a test descriptor system
% with finite stable, finite unstable and infinite eigenvalues of the
% form
%
%   Ex'(t) = Ax(t) + Bu(t),
%     y(t) = Cx(t) + Du(t).
%
% After loading the demo data, the optional parameters are assigned here
% explicitly and the ml_ct_dss_bst function is called with its different
% input interfaces. The dss object version is only called if the System
% Control Toolbox (Matlab) or the Control Package (Octave) is
% installed/loaded.
%
% To show the performance of the model reduction method, the sigma error
% of the full-order and reduced-order model is plotted.
% If the System Control Toolbox is installed, the error bound is plottet.
% In case the System Control Toolbox is installed, also a bode magnitude
% plot of the error system is shown.
%
% See also morlab_demo_ct_ss_bst.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%

%% Initialization
% For demonstration, a random descriptor system example was generated by
% the script morlab_data_desc_infunstab.m and saved in
% morlab_data_desc_infunstab.mat. The number of finite stable, finite
% unstable and infinite eigenvalues as well as the complete size of the
% system is saved in the datainfo structure.
if exist('OCTAVE_VERSION', 'builtin')
    orig_warn = warning('off', 'Octave:data-file-in-path');
    load morlab_data_desc_infunstab.mat;
    warning(orig_warn);
else
    load morlab_data_desc_infunstab.mat;
end

% Get information about installed/loaded toolboxes.
hasControlPkg = size(license('inuse', 'control'), 2);
hasControlTbx = license('test', 'control_toolbox');

%% Construction of the descriptor system structure
% To test the different input-output formats, the struct and state-space
% object shapes of the descriptor system are formulated here.
sys_struct = struct('A', A, 'B', B, 'C', C, 'D', D, 'E', E);

if hasControlPkg || hasControlTbx
    sys_ss = dss(A, B, C, D, E);
end

%% Set of optional parameters
% The default values are mainly taken here, which can be modified.
% Alternative values depending on the system are commented out. Usually for
% using default values the corresponding parameters are not set or empty.
% Also, the function call "opts = ml_morlabopts('ml_ct_dss_bst')" generates
% an empty option struct of the following form.

% Option struct for generalized discrete-time Lyapunov equation solver.
gdlyapopts = struct(...
    'AbsTol' , 0, ...
    'Index'  , Inf, ... % Index = 1
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+01 * datainfo.ni * eps);

% Option struct for Lyapunov solver inside the Riccati solver.
lyapopts1 = struct(...
    'AbsTol' , 0, ...
    'CompTol', 1.0e-02 * sqrt(datainfo.np * eps), ...
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+01 * datainfo.np * eps);

% Option struct for Riccati equation solver.
pcareopts = struct(...
    'AbsTol'  , 0, ...
    'CompTol' , 1.0e-02 * sqrt(datainfo.np * eps), ...
    'Info'    , 0, ... % Info = 1
    'lyapopts', lyapopts1, ...
    'MaxIter' , 100, ...
    'RelTol'  , 1.0e+02 * datainfo.np * eps);


% Option struct for dual Lyapunov equation solver.
lyapopts2 = struct(...
    'AbsTol' , 0, ...
    'CompTol', 1.0e-02 * sqrt(datainfo.np * eps), ...
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+01 * datainfo.np * eps);

% Option struct for decomposition of infinite part.
infdecopts = struct(...
    'AbsTol'   , 0, ...
    'Dimension', -1, ... % Dimension = datainfo.ni
    'Info'     , 0, ... % Info = 1
    'MaxIter'  , 100, ...
    'RelTol'   , 1.0e+02 * datainfo.n * eps, ...
    'RankTol'  , log(datainfo.n) * eps);

% Option struct for decomposition of unstable part.
stabdecopts = struct(...
    'AbsTol'   , 0, ...
    'Dimension', -1, ... % Dimension = datainfo.nu
    'Info'     , 0, ... % Info = 1
    'MaxIter'  , 100, ...
    'RelTol'   , 1.0e+02 * datainfo.n * eps, ...
    'RankTol'  , log(datainfo.n) * eps);

% Option struct for the complete function.
opts = struct(...
    'DecompEig'       , [], ... % DecompEig = 50
    'DecompTol'       , log(datainfo.n) * eps, ...
    'Epsilon'         , 1.0e-03, ...
    'gdlyapopts'      , gdlyapopts, ...
    'ImproperTrunc'   , log(datainfo.n) * eps, ...
    'Index'           , Inf, ... % Index = 3
    'infdecopts'      , infdecopts, ...
    'lyapopts'        , lyapopts2, ...
    'Method'          , 'sr', ... % Method = 'bfsr'
    'Order'           , 10, ...
    'OrderComputation', 'tolerance', ... % OrderComputation = 'order'
    'pcareopts'       , pcareopts, ...
    'stabdecopts'     , stabdecopts, ...
    'Tolerance'       , 1.0e-02);

%% Application of the function
% Here the application of the ml_ct_dss_bst function is shown for different
% interfaces and input-data. The default calls are commented out.

% Application with single matrices.
% [Ar, Br, Cr, Dr, Er, info] = ml_ct_dss_bst(A, B, C, D, E);
[Ar, Br, Cr, Dr, Er, info] = ml_ct_dss_bst(A, B, C, D, E, opts);

% Application with structure.
% [rom_struct, info_struct] = ml_ct_dss_bst(sys_struct);
[rom_struct, info_struct] = ml_ct_dss_bst(sys_struct, opts);

% Application with state-space object.
if hasControlPkg || hasControlTbx
    % [rom_ss, info_ss] = ml_ct_dss_bst(sys_ss);
    [rom_ss, info_ss] = ml_ct_dss_bst(sys_ss, opts);
end

%% Report
% As visualization, a sigmaplot of the error system is made for the
% standard system structures and a bode magnitude plot for the state-
% space objects.

if hasControlTbx
    % Sigmaplot of the error system with error bound.
    figure;
    ml_sigmaplot(sys_struct, rom_struct, -4, 4, 100, ...
        norm(sys_ss, inf) * info.RelErrBound, 'b.');
    legend('(Absolute) error bound', 'Sigma error');
    title({'BST\_DSS (sigmaplot, error system)'; ...
        ['Full order = ' int2str(size(A, 1)) '; ' ...
        'Reduced-order = ' int2str(size(Ar, 1))]});
else
    % Sigmaplot of the error system.
    figure;
    ml_sigmaplot(sys_struct, rom_struct, -4, 4, 100, [], 'b.');
    legend('Sigma error');
    title({'BST\_DSS (sigmaplot, error system)'; ...
        ['Full order = ' int2str(size(A, 1)) '; ' ...
        'Reduced-order = ' int2str(size(Ar, 1))]});
end

if hasControlTbx
    % Bode magnitude plot of the error system.
    bodeopts          = bodeoptions('cstprefs');
    bodeopts.MagUnits = 'abs';
    
    figure;
    bodemag(sys_ss - rom_ss, bodeopts);
    title({'BST\_DSS (Bode magnitude plot, error system)'; ...
        ['Full order = ' int2str(size(A, 1)) '; ' ...
        'Reduced-order = ' int2str(size(Ar, 1))]});
end
