%% HINFBT for standard systems
%
% This demo script contains the application of the H-infinity balanced
% truncation method (ml_ct_ss_hinfbt) on a test standard system with stable
% and unstable eigenvalues of the form
%
%   x'(t) = Ax(t) + Bu(t),
%    y(t) = Cx(t) + Du(t).
%
% After loading the demo data, the optional parameters are assigned here
% explicitly and the ml_ct_ss_hinfbt function is called with its different
% input interfaces. The ss object version is only called if the System
% Control Toolbox (Matlab) or the Control Package (Octave) is
% installed/loaded.
%
% To show the performance of the model reduction method, the sigma error
% of the full-order and reduced-order model is plotted.
% In case the System Control Toolbox is installed, also a bode magnitude
% plot of the error system is shown.
% And if the System Control Toolbox and the Robust Control toolbox are
% installed, a sigma error and bode magnitude plot of the error system
% resulting of the scaled coprime factorization is shown with the
% corresponding error bound.
%
% See also morlab_demo_ct_dss_hinfbt.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%

%% Initialization
% For demonstration, a random standard system example was generated by the
% script morlab_data_std_unstab.m and saved in morlab_data_std_unstab.mat.
% The number of stable and unstable eigenvalues as well as the complete
% size of the system is saved in the datainfo structure.
if exist('OCTAVE_VERSION', 'builtin')
    orig_warn = warning('off', 'Octave:data-file-in-path');
    load morlab_data_std_unstab.mat;
    warning(orig_warn);
else
    load morlab_data_std_unstab.mat;
end

% Get information about installed/loaded toolboxes.
hasControlPkg = size(license('inuse', 'control'), 2);
hasControlTbx = license('test', 'control_toolbox');
hasRobustTbx  = license('test', 'robust_toolbox');

%% Construction of the standard system structure
% To test the different input-output formats, the struct and state-space
% object shapes of the standard system are formulated here.
sys_struct = struct('A', A, 'B', B, 'C', C, 'D', D);

if hasControlPkg || hasControlTbx
    sys_ss = ss(A, B, C, D);
end

%% Construction of the normalized LQG system and GAM computation
% The optimal gamma from the normalized LQG system has been computed in
% advance, since the computation is very time consuming.
% To do the computation from scratch, the following lines have to be
% commented out and the fixed GAM number below commented.


% if hasControlTbx && hasRobustTbx
%     n = size(A, 1);
%     m = size(B, 2);
%     p = size(C, 1);
%
%     sys_normal = ss(A, ...
%         [B, zeros(n, p), B], ...
%         [C; zeros(m, n); C], ...
%         [zeros(p, m), zeros(p, p), zeros(p, m);
%         zeros(m, m), zeros(m, p), eye(m, m);
%         zeros(p, m), eye(p, p), D]);
%
%     [K, CL, GAM, INFO] = hinfsyn(sys_normal, p, m);
% end

GAM = 455.9384;

%% Set of optional parameters
% The default values are mainly taken here, which can be modified.
% Alternative values depending on the system are commented out. Usually for
% using default values the corresponding parameters are not set or empty.
% Also, the function call "opts = ml_morlabopts('ml_ct_ss_hinfbt')" generates an
% empty option struct of the following form.

% Option struct for Lyapunov solver inside the Riccati solver.
lyapopts = struct(...
    'AbsTol' , 0, ...
    'CompTol', 1.0e-02 * sqrt(datainfo.n * eps), ...
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+01 * datainfo.n * eps);

% Option struct for sign function Riccati equation solver.
caredlopts = struct( ...
    'AbsTol' , 0, ...
    'CompTol', 1.0e-02 * sqrt(datainfo.n * eps), ...
    'EqnType', 'both', ...
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+01*datainfo.n*eps);

% Option struct for Riccati equation solver.
careopts = struct(...
    'AbsTol'  , 0, ...
    'Info'    , 0, ... % Info = 1
    'lyapopts', lyapopts, ...
    'MaxIter' , 100, ...
    'RelTol'  , 1.0e+02 * datainfo.n * eps);

% Option struct for Lyapunov or Bernoulli equation solver.
stabmethodopts = struct(...
    'AbsTol' , 0, ...
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+01 * datainfo.n * eps);

% Option structs for the decomposition of unstable part.
stabsignmopts = struct(...
    'AbsTol' , 0, ...
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+02 * datainfo.n * eps);

% Option struct for the complete function.
opts = struct(...
    'Beta'            , 0.1, ...
    'caredlopts'      , caredlopts, ...
    'careopts'        , careopts, ...
    'Gamma'           , GAM, ...
    'Method'          , 'sr', ... % Method = 'bfsr'
    'Order'           , 10, ...
    'OrderComputation', 'tolerance', ... % OrderComputation = 'order'
    'RiccatiSolver'   , 'sign', ... % RiccatiSolver = 'newton'
    'stabmethodopts'  , stabmethodopts, ...
    'stabsignmopts'   , stabsignmopts, ...
    'StabMethod'      , 'abe', ... % StabMethod = 'lyap'
    'Tolerance'       , 1.0e-02, ...
    'UnstabDim'       , -1); % UnstabDim = 8

%% Application of the function
% Here the application of the ml_ct_ss_hinfbt function is shown for different
% interfaces and input-data. The default calls are commented out.

% Application with single matrices.
% [Ar, Br, Cr, Dr, info] = ml_ct_ss_hinfbt(A, B, C, D);
[Ar, Br, Cr, Dr, info] = ml_ct_ss_hinfbt(A, B, C, D, opts);

% Application with structure.
% [rom_struct, info_struct] = ml_ct_ss_hinfbt(sys_struct);
[rom_struct, info_struct] = ml_ct_ss_hinfbt(sys_struct, opts);

% Application with state-space object.
if hasControlPkg || hasControlTbx
    % [rom_ss, info_ss] = ml_ct_ss_hinfbt(sys_ss);
    [rom_ss, info_ss] = ml_ct_ss_hinfbt(sys_ss, opts);
end

%% Report
% As visualization, a sigmaplot of the error system of the (right) coprime
% factors is made for the standard system structures and a bode magnitude
% plot for the state-space objects.

% Sigmaplot of the error system.
figure;
ml_sigmaplot(sys_struct, rom_struct, -4, 4, 200);
legend('Sigma error');
title({'HINFBT (sigmaplot, error system)'; ...
    ['Full order = ' int2str(size(A, 1)) '; ' ...
    'Reduced-order = ' int2str(size(Ar, 1))]});

if hasControlTbx
    % Bode magnitude plots of the error system.
    bodeopts          = bodeoptions('cstprefs');
    bodeopts.MagUnits = 'abs';
    
    figure;
    bodemag(sys_ss - rom_ss, bodeopts);
    title({'HINFBT (Bode magnitude plot, error system)'; ...
        ['Full order = ' int2str(size(A, 1)) '; ' ...
        'Reduced-order = ' int2str(size(Ar, 1))]});
    
    if hasRobustTbx
        % Coprime factorization.
        beta = sqrt(1 - 1 / info.Gamma^2);
        
        [~, sysINFO] = ncfmr(beta * sys_ss, size(sys_ss.a, 1));
        [~, romINFO] = ncfmr(beta * rom_ss, size(rom_ss.a, 1));
        
        sys_GR = struct(...
            'A', sysINFO.GR.A, ...
            'B', sysINFO.GR.B, ...
            'C', sysINFO.GR.C, ...
            'D', sysINFO.GR.D);
        
        rom_GR = struct(...
            'A', romINFO.GR.A, ...
            'B', romINFO.GR.B, ...
            'C', romINFO.GR.C, ...
            'D', romINFO.GR.D);
        
        % Sigmaplot of the coprime error system.
        figure;
        ml_sigmaplot(sys_GR, rom_GR, -4, 4, 100, ...
            beta * info.AbsErrBound, 'b.');
        legend('Scaled error bound', 'Sigma error of coprime factors ');
        title({'HINFBT (sigmaplot, coprime error system)'; ...
            ['Full order = ' int2str(size(A, 1)) '; ' ...
            'Reduced-order = ' int2str(size(Ar, 1))]});
        
        % Bode magnitude plot of the coprime error system.
        bodeopts          = bodeoptions('cstprefs');
        bodeopts.MagUnits = 'abs';
        
        figure;
        bodemag(sysINFO.GR - romINFO.GR, bodeopts);
        title({'HINFBT (Bode magnitude plot, coprime error system)'; ...
            ['Full order = ' int2str(size(A, 1)) '; ' ...
            'Reduced-order = ' int2str(size(Ar, 1))]});
    end
end
