%% MT for standard systems
%
% This demo script contains the application of the modal truncation
% method (ml_ct_ss_mt) on a test standard system with stable and unstable
% eigenvalues of the form
%
%   x'(t) = Ax(t) + Bu(t),
%    y(t) = Cx(t) + Du(t).
%
% After loading the demo data, the optional parameters are assigned here
% explicitly and the ml_ct_ss_mt function is called with its different input
% interfaces. The ss object version is only called if the System Control
% Toolbox (Matlab) or the Control Package (Octave) is installed/loaded.
%
% To show the performance of the model reduction method, the sigma error
% of the full-order and reduced-order model is plotted.
% In case the System Control Toolbox is installed, also a bode magnitude
% plot of the full-order and reduced-order systems is shown.
%
% See also morlab_demo_ct_dss_mt.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%

%% Initialization
% For demonstration, a random standard system example was generated by the
% script morlab_data_std_unstab.m and saved in morlab_data_std_unstab.mat.
% The number of stable and unstable eigenvalues as well as the complete
% size of the system is saved in the datainfo structure.
if exist('OCTAVE_VERSION', 'builtin')
    orig_warn = warning('off', 'Octave:data-file-in-path');
    load morlab_data_std_unstab.mat;
    warning(orig_warn);
else
    load morlab_data_std_unstab.mat;
end

% Get information about installed/loaded toolboxes.
hasControlPkg = size(license('inuse', 'control'), 2);
hasControlTbx = license('test', 'control_toolbox');

%% Construction of the standard system structure
% To test the different input-output formats, the struct and state-space
% object shapes of the standard system are formulated here.
sys_struct = struct('A', A, 'B', B, 'C', C, 'D', D);

if hasControlPkg || hasControlTbx
    sys_ss = ss(A, B, C, D);
end

%% Set of optional parameters
% The default values are mainly taken here, which can be modified.
% Alternative values depending on the system are commented out. Usually for
% using default values the corresponding parameters are not set or empty.
% Also, the function call "opts = ml_morlabopts('ml_ct_ss_mt')" generates an
% empty option struct of the following form.

% Option structs for the decomposition of the eigenvalues.
signmopts = struct(...
    'AbsTol' , 0, ...
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+02 * datainfo.n * eps);

sylvopts = struct(...
    'AbsTol' , 0, ...
    'Info'   , 0, ... % Info = 1
    'MaxIter', 100, ...
    'RelTol' , 1.0e+01 * datainfo.n * eps);

% Option struct for the complete function.
opts = struct(...
    'Alpha'    , -1.0, ...
    'signmopts', signmopts, ...
    'sylvopts' , sylvopts);

%% Application of the function
% Here the application of the ml_ct_ss_mt function is shown for different
% interfaces and input-data. The default calls are commented out.

% Application with single matrices.
% [Ar, Br, Cr, Dr, info] = ml_ct_ss_mt(A, B, C, D);
[Ar, Br, Cr, Dr, info] = ml_ct_ss_mt(A, B, C, D, opts);

% Application with structure.
% [rom_struct, info_struct] = ml_ct_ss_mt(sys_struct);
[rom_struct, info_struct] = ml_ct_ss_mt(sys_struct, opts);

% Application with state-space object.
if hasControlPkg || hasControlTbx
    % [rom_ss, info_ss] = ml_ct_ss_mt(sys_ss);
    [rom_ss, info_ss] = ml_ct_ss_mt(sys_ss, opts);
end

%% Report
% As visualization, a sigmaplot of the error system is made for the
% standard system structures and a bode magnitude plot for the state-
% space objects.

% Sigmaplot.
figure;
ml_sigmaplot(sys_struct, rom_struct, -4, 4, 100, [], 'b.');
legend('Sigma error');
title({'MT (sigmaplot, error system)'; ...
    ['Full order = ' int2str(size(A, 1)) '; ' ...
    'Reduced-order = ' int2str(size(Ar, 1))]});

if hasControlTbx
    % Bode magnitude plot of full- and reduced-order systems.
    bodeopts          = bodeoptions('cstprefs');
    bodeopts.MagUnits = 'abs';
    
    figure;
    bodemag(sys_ss, rom_ss, bodeopts);
    title({'MT (Bode magnitude plot)'; ...
        ['Full order = ' int2str(size(A, 1)) '; ' ...
        'Reduced-order = ' int2str(size(Ar, 1))]});
end
