function [varargout] = ml_ct_dss_brbt(varargin)
%ML_CT_DSS_BRBT Bounded-real balanced truncation for descriptor systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_BRBT(A, B, C, D, E)
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_BRBT(A, B, C, D, E, opts)
%
%   [rom, info] = ML_CT_DSS_BRBT(sys)
%   [rom, info] = ML_CT_DSS_BRBT(sys, opts)
%
% DESCRIPTION:
%   This function computes the generalized bounded-real balanced truncation
%   for a bounded-real descriptor system of the form
%
%       E*x'(t) = A*x(t) + B*u(t),                                      (1)
%          y(t) = C*x(t) + D*u(t).                                      (2)
%
%   Therefore, first an additive decomposition of the system is performed
%   using the matrix disk function, such that
%
%            [ Ei  0 ]       [ Ai  0 ]                        [ Ci ]
%       E2 = [       ], A2 = [       ], B2 = [ Bi, Bp ], C2 = [    ],
%            [ 0  Ep ]       [ 0  Ap ]                        [ Cp ]
%
%   with (Ei, Ai, Bi, Ci, D) belonging to the polynomial part and
%   (Ep, Ap, Bp, Cp, 0) belonging to the strictly proper part.
%   Now, the two generalized bounded-real Riccati equations
%
%       Ap*Pp*Ep' + Ep*Pp*Ap' + Bp*Bp'
%           + (Ep*Pp*Cp' + Bp*M') * inv(Rb) * (Ep*Pp*Cp' + Bp*M')' = 0,
%       Ap'*Qp*Ep + Ep'*Qp*Ap + Cp'*Cp
%           + (Bp'*Qp*Ep + M'*Cp)' * inv(Rc) * (Bp'*Qp*Ep + M'*Cp) = 0,
%
%   are solved for the reduction of the strictly proper part, with
%
%       Rb = I - M*M'   and
%       Rc = I - M'*M,
%
%   where M = D - Ci * inv(Ai) * Bi. Also, the two generalized
%   discrete-time Lyapunov equations
%
%       Ai*Pi*Ai' - Ei*Pi*Ei' - Bi*Bi' = 0,
%       Ai'*Qi*Ai - Ei'*Qi*Ei - Ci'*Ci = 0
%
%   are solved for the reduction of the polynomial part. As result, a
%   reduced-order system of the form
%
%       Er*x'(t) = Ar*x(t) + Br*u(t),                                   (3)
%           y(t) = Cr*x(t) + Dr*u(t)                                    (4)
%
%   is computed, such that for the original transfer function G with the
%   spectral factors W'*W = I - G'*G and  V*V' = I - G*G', and the
%   reduced-order transfer function Gr with an r-th order strictly proper
%   part and the spectral factors Wr'*Wr = I - Gr'*Gr and
%   Vr*Vr' = I - Gr*Gr' it holds
%
%       max(||[G - Gr; W - Wr]||_{\infty}, ||[G - Gr; V - Vr]||_{\infty})
%       <= 2 * (Hsvp(r+1) + ... + Hsvp(n)),
%
%   with Hsvp, a vector containing the proper characteristic bounded-real
%   singular values of the system.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   E    - matrix from (1) with dimensions n x n
%   sys  - structure or state-space object, containing the descriptor
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | DecompEig       | positive scalar, overestimation of the absolute   |
%   |                 | value of the largest finite eigenvalue of s*E - A,|
%   |                 | if set, replaces the computation with DecompTol   |
%   |                 | (default [])                                      |
%   +-----------------+---------------------------------------------------+
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | gdlyapopts      | structure, containing the optional parameters for |
%   |                 | the computation of the generalized discrete-time  |
%   |                 | Lyapunov equations, see ml_gdlyap_smith_fac       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | ImproperTrunc   | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest proper Hankel singular value of the       |
%   |                 | system to truncate the improper part              |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Index           | nonnegative integer, index of the descriptor      |
%   |                 | system used to set an upper bound on the size of  |
%   |                 | the reduced improper part, Inf if unknown         |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, see ml_disk and ml_getqz       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   |                 | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsvp) + Ni)                |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   |                 | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | pcareopts       | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | positive Riccati equation, see ml_pcare_nwt_fac   |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   |                 | computation of the size of the reduced-order model|
%   |                 | by an absolute error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   Er   - matrix of (3) with dimensions r x r
%   rom  - structure or state-space object, containing the reduced-order
%          descriptor system:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   |                 | reduced-order model in H-infinity norm            |
%   +-----------------+---------------------------------------------------+
%   | Hsvi            | a vector, containing the computed Hankel singular |
%   |                 | values of the improper part of the system         |
%   +-----------------+---------------------------------------------------+
%   | Hsvp            | a vector, containing the computed characteristic  |
%   |                 | bounded-real singular values of the proper part   |
%   |                 | of the system                                     |
%   +-----------------+---------------------------------------------------+
%   | infoGADTF       | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | infinite and finite parts, see ml_ct_dss_adtf     |
%   +-----------------+---------------------------------------------------+
%   | infoGDLYAP_C    | structure, containing information about the       |
%   |                 | generalized discrete-time Lyapunov equation solver|
%   |                 | for the improper controllability Gramian,         |
%   |                 | see ml_gdlyap_smith_fac                           |
%   +-----------------+---------------------------------------------------+
%   | infoGDLYAP_O    | structure, containing information about the       |
%   |                 | generalized discrete-time Lyapunov equation solver|
%   |                 | for the improper observability Gramian,           |
%   |                 | see ml_gdlyap_smith_fac                           |
%   +-----------------+---------------------------------------------------+
%   | infoPCARE_C     | structure, containing information about the       |
%   |                 | continuous-time algebraic positive Riccati        |
%   |                 | equation solver for the controllability Gramian,  |
%   |                 | see ml_pcare_nwt_fac                              |
%   +-----------------+---------------------------------------------------+
%   | infoPCARE_O     | structure, containing information about the       |
%   |                 | continuous-time algebraic positive Riccati        |
%   |                 | equation solver for the observability Gramian,    |
%   |                 | see ml_pcare_nwt_fac                              |
%   +-----------------+---------------------------------------------------+
%   | Ni              | Dimension of the improper part in the reduced-    |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | Np              | Dimension of the proper part in the reduced-order |
%   |                 | model                                             |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   T. Reis, T. Stykel, Positive real and bounded real balancing for model
%   reduction of descriptor systems, Internat. J. Control 83 (1) (2010)
%   74--88. https://doi.org/10.1080/00207170903100214
%
% See also ml_ct_ss_brbt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_dss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 6);
else
    nargoutchk(0, 2);
end

% Fill the D matrix.
if isempty(sys.D)
    sys.D = zeros(size(sys.C, 1), size(sys.B, 2));
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'gdlyapopts')
    assert(isa(opts.gdlyapopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.gdlyapopts has to be a struct!');
else
    opts.gdlyapopts = struct();
end

if ml_field_set_to_value(opts, 'Method')
    if strcmpi(opts.Method, 'bfsr')
        mor_method = @ml_ct_dss_bfsr;
    elseif strcmpi(opts.Method, 'sr')
        mor_method = @ml_ct_dss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
else
    mor_method = @ml_ct_dss_sr;
end

if ml_field_set_to_value(opts, 'OrderComputation')
    if strcmpi(opts.OrderComputation, 'order')
        tselect = 0;
    elseif strcmpi(opts.OrderComputation, 'tolerance')
        tselect = 3;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
else
    tselect = 3;
end

if ml_field_set_to_value(opts, 'pcareopts')
    assert(isa(opts.pcareopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.pcareopts has to be a struct!');
else
    opts.pcareopts = struct();
end

if ml_field_set_to_value(opts, 'stabdecopts')
    assert(isa(opts.stabdecopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.stabdecopts has to be a struct!');
else
    opts.stabdecopts = struct();
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into finite and infinite parts.
opts.stabdecopts.Dimension = 0;
[sys, infoGADTF]           = ml_ct_dss_adtf(sys, opts);

ninf = size(sys.Ainf, 1);

% Assign information about additive decomposition.
info.infoGADTF = infoGADTF;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVE MATRIX EQUATIONS.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of the constant polynomial part.
if ninf > 0
    M = sys.D - sys.Cinf * (sys.Ainf \ sys.Binf);
else
    M = sys.D;
end

if size(sys.A, 1) > 0
    % Computation of matrices for Riccati equations.
    Rb = eye(size(sys.C, 1)) - M * M';
    Lb = chol(Rb);
    Rc = eye(size(sys.B, 2)) - M' * M;
    Lc = chol(Rc);
    
    Ab = sys.A + sys.B * ((M' / Rb) * sys.C);
    Ac = sys.A + sys.B * ((Rc \ M') * sys.C);
    B  = sys.B / Lc;
    C  = Lb' \ sys.C;
    
    % Computation of the full-rank factors of the proper Gramians.
    [R, infoPCARE_C] = ml_pcare_nwt_fac(Ab', C', B', sys.E', ...
        opts.pcareopts);
    
    [L, infoPCARE_O] = ml_pcare_nwt_fac(Ac, B, C, sys.E, ...
        opts.pcareopts);
    
    % Assign information about Riccati equation solver.
    info.infoPCARE_C = infoPCARE_C;
    info.infoPCARE_O = infoPCARE_O;
else
    [R, L] = deal([]);
end

% Computation of the full-rank factors of the improper Gramians.
if ninf > 0
    [Rinf, infoGDLYAP_C] = ml_gdlyap_smith_fac(sys.Ainf', sys.Binf', ...
        sys.Einf', opts.gdlyapopts);
    [Linf, infoGDLYAP_O] = ml_gdlyap_smith_fac(sys.Ainf, sys.Cinf, ...
        sys.Einf, opts.gdlyapopts);
    
    % Assign information about improper Lyapunov equation solver.
    info.infoGDLYAP_C = infoGDLYAP_C;
    info.infoGDLYAP_O = infoGDLYAP_O;
else
    [Rinf, Linf] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[sys, hsvp, hsvi] = mor_method(sys, R, Rinf, L, Linf, tselect, opts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = blkdiag(sys.A, sys.Ainf);
        varargout{2} = [sys.B; sys.Binf];
        varargout{3} = [sys.C, sys.Cinf];
        varargout{4} = sys.D;
        varargout{5} = blkdiag(sys.E, sys.Einf);
        
    case 1
        varargout{1} = struct(...
            'A', blkdiag(sys.A, sys.Ainf), ...
            'B', [sys.B; sys.Binf], ...
            'C', [sys.C, sys.Cinf], ...
            'D', sys.D, ...
            'E', blkdiag(sys.E, sys.Einf));
        
    case 2
        varargout{1} = dss(...
            blkdiag(sys.A, sys.Ainf), ...
            [sys.B; sys.Binf], ...
            [sys.C, sys.Cinf], ...
            sys.D, ...
            blkdiag(sys.E, sys.Einf));
end

% Assign output information.
nf = size(sys.A, 1);

info.AbsErrBound = 2 * sum(hsvp(nf+1:end));
info.Hsvi        = hsvi;
info.Hsvp        = hsvp;
info.Ni          = size(sys.Ainf, 1);
info.Np          = nf;

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

if ioformat == 0
    varargout{6} = info;
else
    varargout{2} = info;
end
