function [varargout] = ml_ct_dss_flbt(varargin)
%ML_CT_DSS_FLBT Frequency-limited balanced truncation for desc. systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_FLBT(A, B, C, D, E)
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_FLBT(A, B, C, D, E, opts)
%
%   [rom, info] = ML_CT_DSS_FLBT(sys)
%   [rom, info] = ML_CT_DSS_FLBT(sys, opts)
%
% DESCRIPTION:
%   This function computes the generalized frequency-limited balanced
%   truncation for a descriptor system of the form
%
%       E*x'(t) = A*x(t) + B*u(t),                                      (1)
%          y(t) = C*x(t) + D*u(t).                                      (2)
%
%   Therefore, first an additive decomposition of the system is performed
%   using the matrix disk function, such that
%
%            [ Ei  0 ]       [ Ai  0 ]                        [ Ci ]
%       E2 = [       ], A2 = [       ], B2 = [ Bi, Bp ], C2 = [    ],
%            [ 0  Ep ]       [ 0  Ap ]                        [ Cp ]
%
%   with (Ei, Ai, Bi, Ci, D) belonging to the polynomial part and
%   (Ep, Ap, Bp, Cp, 0) belonging to the strictly proper part.
%   Now, the two generalized continuous-time Lyapunov equations
%
%       Ap*Pp*Ep' + Ep*Pp*Ap' + BF*Bp' + Bp*BF' = 0,
%       Ap'*Qp*Ep + Ep'*Qp*Ap + Cp'*CF + CF'*Cp = 0,
%
%   where BF and CF are frequency-dependent matrices, are solved for the
%   reduction of the strictly proper part, and the two generalized
%   discrete-time Lyapunov equations
%
%       Ai*Pi*Ai' - Ei*Pi*Ei' - Bi*Bi' = 0,
%       Ai'*Qi*Ai - Ei'*Qi*Ei - Ci'*Ci = 0
%
%   are solved for the reduction of the polynomial part. As result, a
%   reduced-order system of the form
%
%       Er*x'(t) = Ar*x(t) + Br*u(t),                                   (3)
%           y(t) = Cr*x(t) + Dr*u(t)                                    (4)
%
%   is computed. For enforcing stability in the reduced-order model, the
%   modified Gramian approach can be used, which also gives a global error
%   bound of the form
%
%       ||G - Gr||_{\infty} <= 2*||JB||*||JC||(Hsvp(r+1) + ... + Hsvp(n)),
%
%   with Hsvp, a vector containing the proper frequency-limited Hankel
%   singular values of the system.
%
%   Note: For unstable systems, an additional additive decomposition into
%         the stable and anti-stable parts is performed and then only the
%         stable part will be reduced.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   E    - matrix from (1) with dimensions n x n
%   sys  - structure or state-space object, containing the descriptor
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | DecompEig       | positive scalar, overestimation of the absolute   |
%   |                 | value of the largest finite eigenvalue of s*E - A,|
%   |                 | if set, replaces the computation with DecompTol   |
%   |                 | (default [])                                      |
%   +-----------------+---------------------------------------------------+
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | FreqRange       | nonnegative vector, frequency intervals such that |
%   |                 | [w(1), w(2)] ... [w(2k-1), w(2k)] are approximated|
%   |                 | (default [0, 1.0e+03])                            |
%   +-----------------+---------------------------------------------------+
%   | gdlyapopts      | structure, containing the optional parameters for |
%   |                 | the computation of the generalized discrete-time  |
%   |                 | Lyapunov equations, see ml_gdlyap_smith_fac       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | ImproperTrunc   | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest proper Hankel singular value of the       |
%   |                 | system to truncate the improper part              |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Index           | nonnegative integer, index of the descriptor      |
%   |                 | system used to set an upper bound on the size of  |
%   |                 | the reduced improper part, Inf if unknown         |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, see ml_disk and ml_getqz       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | lyapdlopts      | structure, containing the optional parameters for |
%   |                 | the computation of the generalized continuous-time|
%   |                 | Lyapunov equations, see ml_lyapdl_sgn_ldl if      |
%   |                 | ModGramian = 0 and ml_lyapdl_sgn_fac if           |
%   |                 | ModGramian = 1                                    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   |                 | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | ModGramian      | {0, 1}, used to disable/enable the modified       |
%   |                 | Gramian approach                                  |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsvp)) + Nu + Ni)          |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   |                 | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using rel. tolerance for the hsv   |
%   |                 |  'sum'       - using rel. tolerance for sum of hsv|
%   |                 | (default 'sum')                                   |
%   +-----------------+---------------------------------------------------+
%   | stabdecopts     | structure, containing the optional parameters for |
%   |                 | the decomposition of the stable and unstable parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, see ml_disk and ml_getqz       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   |                 | computation of the size of the reduced-order model|
%   |                 | if 'tolerance' or 'sum' is set for                |
%   |                 | OrderComputation                                  |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   Er   - matrix of (3) with dimensions r x r
%   rom  - structure or state-space object, containing the reduced-order
%          descriptor system:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   |                 | reduced-order model in H-infinity norm, only for  |
%   |                 | the modified Gramian approach                     |
%   +-----------------+---------------------------------------------------+
%   | Hsvi            | a vector, containing the computed Hankel singular |
%   |                 | values of the improper part of the system         |
%   +-----------------+---------------------------------------------------+
%   | Hsvp            | a vector, containing the computed Hankel singular |
%   |                 | values of the proper part of the system           |
%   +-----------------+---------------------------------------------------+
%   | infoGADTF       | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | infinite, finite stable and finite anti-stable    |
%   |                 | parts, see ml_ct_dss_adtf                         |
%   +-----------------+---------------------------------------------------+
%   | infoGDLYAP_C    | structure, containing information about the       |
%   |                 | generalized discrete-time Lyapunov equation solver|
%   |                 | for the improper controllability Gramian,         |
%   |                 | see ml_gdlyap_smith_fac                           |
%   +-----------------+---------------------------------------------------+
%   | infoGDLYAP_O    | structure, containing information about the       |
%   |                 | generalized discrete-time Lyapunov equation solver|
%   |                 | for the improper observability Gramian,           |
%   |                 | see ml_gdlyap_smith_fac                           |
%   +-----------------+---------------------------------------------------+
%   | infoLYAPDL      | structure, containing information about the       |
%   |                 | continuous-time dual Lyapunov equations solver,   |
%   |                 | see ml_lyapdl_sgn_ldl or ml_lyapdl_sgn_fac        |
%   +-----------------+---------------------------------------------------+
%   | Ni              | Dimension of the improper part in the reduced-    |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | Np              | Dimension of the proper part in the reduced-order |
%   |                 | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Nu              | Dimension of the unstable part in the reduced-    |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%
% See also ml_ct_dss_bt, ml_ct_ss_flbt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_dss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 6);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'FreqRange')
    ml_assert_nonnegvector(opts.FreqRange, 'opts.FreqRange');
    
    assert(mod(length(opts.FreqRange), 2) == 0, ...
        'MORLAB:data', ...
        'The parameter opts.FreqRange has to be an even vector!');
    
    opts.FreqRange = sort(opts.FreqRange);
else
    opts.FreqRange = [0, 1.0e+03];
end

if ml_field_set_to_value(opts, 'gdlyapopts')
    assert(isa(opts.gdlyapopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.gdlyapopts has to be a struct!');
else
    opts.gdlyapopts = struct();
end

if ml_field_set_to_value(opts, 'lyapdlopts')
    assert(isa(opts.lyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.lyapdlopts has to be a struct!');
else
    opts.lyapdlopts = struct();
end

if ml_field_set_to_value(opts, 'Method')
    if strcmpi(opts.Method, 'bfsr')
        mor_method = @ml_ct_dss_bfsr;
    elseif strcmpi(opts.Method, 'sr')
        mor_method = @ml_ct_dss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
else
    mor_method = @ml_ct_dss_sr;
end

if ml_field_set_to_value(opts, 'ModGramian')
    ml_assert_boolean(opts.ModGramian, 'opts.ModGramian');
else
    opts.ModGramian = 0;
end

if ml_field_set_to_value(opts, 'OrderComputation')
    if strcmpi(opts.OrderComputation, 'order')
        tselect = 0;
    elseif strcmpi(opts.OrderComputation, 'tolerance')
        tselect = 1;
    elseif strcmpi(opts.OrderComputation, 'sum')
        tselect = 2;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
else
    tselect = 2;
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into unstable, finite and infinite parts.
[sys, infoGADTF] = ml_ct_dss_adtf(sys, opts);

ninf = size(sys.Ainf, 1);

% Assign information about additive decomposition.
info.infoGADTF = infoGADTF;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVE MATRIX EQUATIONS.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = size(sys.A, 1);
m = size(sys.B, 2);
p = size(sys.C, 1);

% Computation of the full-rank factors of the proper Gramians.
if n > 0
    % Compute frequency-dependent right-hand side.
    if opts.FreqRange(1) == 0
        F      = -sys.E \ sys.A - 1i * opts.FreqRange(2) * eye(n);
        kstart = 2;
    else
        F      = eye(n);
        kstart = 1;
    end
    
    for k = kstart:1:length(opts.FreqRange)/2
        F = F * ((sys.A + 1i * opts.FreqRange(2*k-1) * sys.E) ...
            \ (sys.A + 1i * opts.FreqRange(2*k) * sys.E));
    end
    
    F  = (1i / pi) * logm(F);
    BF = sys.E * real(F * (sys.E \ sys.B));
    CF = real(sys.C * F);
    BQ = [zeros(m), eye(m); eye(m), zeros(m)];
    CQ = [zeros(p), eye(p); eye(p), zeros(p)];
    
    if opts.ModGramian
        % Modified Gramian approach.
        [BF, Y] = ml_compress_ldl([sys.B, BF], BQ, eps, 'col');
        Y       = abs(diag(Y));
        JB      = norm(diag(1 ./ sqrt(Y)) * BF' * sys.B, 2);
        BF      = BF * diag(sqrt(Y));
        
        [CF, Y] = ml_compress_ldl([sys.C; CF], CQ, eps, 'row');
        Y       = abs(diag(Y));
        JC      = norm(sys.C * CF' * diag(1 ./ sqrt(Y)), 2);
        CF      = diag(sqrt(Y)) * CF;
        
        [R, L, infoLYAPDL] = ml_lyapdl_sgn_fac(sys.A, BF, CF, sys.E, ...
            opts.lyapdlopts);
    else
        % Classical method.
        [R, RY, L, LY, infoLYAPDL] = ml_lyapdl_sgn_ldl(sys.A, ...
            [sys.B, BF], BQ, [sys.C; CF], CQ, sys.E, opts.lyapdlopts);
        
        RY  = diag(RY);
        idx = (RY > 0);
        R   = diag(sqrt(RY(idx))) * R(idx, :);
        
        LY  = diag(LY);
        idx = (LY > 0);
        L   = diag(sqrt(LY(idx))) * L(idx, :);
    end
    
    % Assig information about Lyapunov equation solver.
    info.infoLYAPDL = infoLYAPDL;
else
    [R, L] = deal([]);
end

% Computation of the full-rank factors of the improper Gramians.
if ninf > 0
    [Rinf, infoGDLYAP_C] = ml_gdlyap_smith_fac(sys.Ainf', sys.Binf', ...
        sys.Einf', opts.gdlyapopts);
    [Linf, infoGDLYAP_O] = ml_gdlyap_smith_fac(sys.Ainf, sys.Cinf, ...
        sys.Einf, opts.gdlyapopts);
    
    % Assign information about improper Lyapunov equation solver.
    info.infoGDLYAP_C = infoGDLYAP_C;
    info.infoGDLYAP_O = infoGDLYAP_O;
else
    [Rinf, Linf] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[sys, hsvp, hsvi] = mor_method(sys, R, Rinf, L, Linf, tselect, opts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = blkdiag(sys.A, sys.Au, sys.Ainf);
        varargout{2} = [sys.B; sys.Bu; sys.Binf];
        varargout{3} = [sys.C, sys.Cu, sys.Cinf];
        varargout{4} = sys.D;
        varargout{5} = blkdiag(sys.E, sys.Eu, sys.Einf);
        
    case 1
        varargout{1} = struct(...
            'A', blkdiag(sys.A, sys.Au, sys.Ainf), ...
            'B', [sys.B; sys.Bu; sys.Binf], ...
            'C', [sys.C, sys.Cu, sys.Cinf], ...
            'D', sys.D, ...
            'E', blkdiag(sys.E, sys.Eu, sys.Einf));
        
    case 2
        varargout{1} = dss(...
            blkdiag(sys.A, sys.Au, sys.Ainf), ...
            [sys.B; sys.Bu; sys.Binf], ...
            [sys.C, sys.Cu, sys.Cinf], ...
            sys.D, ...
            blkdiag(sys.E, sys.Eu, sys.Einf));
end

% Assign output information.
nf = size(sys.A, 1);

if opts.ModGramian
    info.AbsErrBound = 2 * JB * JC * sum(hsvp(nf+1:end));
end

info.Hsvi = hsvi;
info.Hsvp = hsvp;
info.Ni   = size(sys.Ainf, 1);
info.Np   = nf;
info.Nu   = size(sys.Au, 1);

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

if ioformat == 0
    varargout{6} = info;
else
    varargout{2} = info;
end
