function [varargout] = ml_ct_dss_mt(varargin)
%ML_CT_DSS_MT Modal truncation for descriptor systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_MT(A, B, C, D, E)
%   [Ar, Br, Cr, Dr, Er, info] = ML_CT_DSS_MT(A, B, C, D, E, opts)
%
%   [rom, info] = ML_CT_DSS_MT(sys)
%   [rom, info] = ML_CT_DSS_MT(sys, opts)
%
% DESCRIPTION:
%   This function computes the generalized modal truncation for a
%   descriptor system of the form
%
%       E*x'(t) = A*x(t) + B*u(t),                                      (1)
%          y(t) = C*x(t) + D*u(t).                                      (2)
%
%   Therefore, a block diagonalization of the matrix pencil s*E - A is
%   performed using the matrix disk function, such that
%
%            [ Ei 0  0  ]       [ Ai 0  0  ]
%       E2 = [ 0  E0 0  ], A2 = [ 0  A0 0  ],
%            [ 0  0  E1 ]       [ 0  0  A1 ]
%
%                                 [ Ci ]
%       B2 = [ Bi, B0, B1 ], C2 = [ C0 ],
%                                 [ C1 ]
%
%   where the matrix pencil s*Ei - Ai contains all infinite eigenvalues
%   and s*E0 - A0 the finite eigenvalues with the real part larger than a
%   given alpha. As result, the reduced-order system is given by
%
%       Er*x'(t) = Ar*x(t) + Br*u(t),                                   (3)
%           y(t) = Cr*x(t) + Dr*u(t),                                   (4)
%
%   with
%
%            [ E0  0 ]       [ A0  0 ]                        [ C0 ]
%       Er = [       ], A2 = [       ], B2 = [ B0, Bi ], C2 = [    ].
%            [ 0  Ei ]       [ 0  Ai ]                        [ Ci ]
%
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   E    - matrix from (1) with dimensions n x n
%   sys  - structure or state-space object, containing the descriptor
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Alpha           | scalar, such that all finite eigenvalues with the |
%   |                 | real part smaller than Alpha are truncated        |
%   |                 | (default -1.0)                                    |
%   +-----------------+---------------------------------------------------+
%   | DecompEig       | positive scalar, overestimation of the absolute   |
%   |                 | value of the largest finite eigenvalue of s*E - A,|
%   |                 | if set, replaces the computation with DecompTol   |
%   |                 | (default [])                                      |
%   +-----------------+---------------------------------------------------+
%   | DecompTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest singular value of E to determine the      |
%   |                 | smallest non-quasi-zero singular value of E       |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | diskopts        | structure, containing the optional parameters for |
%   |                 | the inverse free iteration of the disk function   |
%   |                 | method, see ml_disk                               |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | infdecopts      | structure, containing the optional parameters for |
%   |                 | the decomposition of the finite and infinite parts|
%   |                 | of the system using the disk function and subspace|
%   |                 | extraction method, see ml_disk and ml_getqz       |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | RankTol         | nonnegative scalar, tolerance used for the        |
%   |                 | determination of deflating subspaces              |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   Er   - matrix of (3) with dimensions r x r
%   rom  - structure or state-space object, containing the reduced-order
%          descriptor system:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | infoDISK        | structure, containing information about the disk  |
%   |                 | function method used for the modal truncation of  |
%   |                 | the system, see ml_disk                           |
%   +-----------------+---------------------------------------------------+
%   | infoGADTF       | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | infinite and finite parts, see ml_ct_dss_adtf     |
%   +-----------------+---------------------------------------------------+
%   | N               | Dimension of the finite part in the reduced-order |
%   |                 | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Ni              | Dimension of the infinite part in the reduced-    |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, E. S. Quintana-Orti, Model reduction based on spectral
%   projection methods, in: P. Benner, V. Mehrmann, D. Sorensen (Eds.),
%   Dimension Reduction of Large-Scale Systems, Vol. 45 of Lect. Notes
%   Comput. Sci. Eng., Springer, Berlin/Heidelberg, Germany, 2005,
%   pp. 5--45. https://doi.org/10.1007/3-540-27909-1_1
%
% See also ml_ct_ss_mt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_dss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 6);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'Alpha')
    ml_assert_scalar(opts.Alpha, 'opts.Alpha');
else
    opts.Alpha = -1.0;
end

if ml_field_set_to_value(opts, 'diskopts')
    assert(isa(opts.diskopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.diskopts has to be a struct!');
else
    opts.diskopts = struct();
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into finite and infinite parts.
opts.stabdecopts.Dimension = 0;
[sys, infoGADTF]           = ml_ct_dss_adtf(sys, opts);

n = size(sys.A, 1);

% Assign information about additive decomposition.
info.infoGADTF = infoGADTF;

% Decomposition of desired and undesired eigenvalues.
if n > 0
    [Aspace, Espace, infoDISK] = ml_disk(sys.A + (1 - opts.Alpha) * ...
        sys.E, sys.A - (1 + opts.Alpha) * sys.E, opts.diskopts);
    [Q, ~, n1]                 = ml_getqz(sys.A, sys.E, Aspace, Espace, ...
        opts);
    
    if (n1 > 0) && (n1 < n)
        opts.Dimension = n - n1;
        [~, V, ns]     = ml_getqz(sys.A, sys.E, Espace, Aspace, opts);
        
        sys.A = Q(: , n1+1:end)' * sys.A * V(: , 1:ns);
        sys.E = Q(: , n1+1:end)' * sys.E * V(: , 1:ns);
        sys.B = Q(: , n1+1:end)' * sys.B;
        sys.C = sys.C * V(: , 1:ns);
    elseif n1 == n
        [sys.A, sys.B, sys.C, sys.E] = deal([]);
    end
    
    % Assign information about the disk function.
    info.infoDISK = infoDISK;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = blkdiag(sys.A, sys.Ainf);
        varargout{2} = [sys.B; sys.Binf];
        varargout{3} = [sys.C, sys.Cinf];
        varargout{4} = sys.D;
        varargout{5} = blkdiag(sys.E, sys.Einf);
        
    case 1
        varargout{1} = struct(...
            'A', blkdiag(sys.A, sys.Ainf), ...
            'B', [sys.B; sys.Binf], ...
            'C', [sys.C, sys.Cinf], ...
            'D', sys.D, ...
            'E', blkdiag(sys.E, sys.Einf));
        
    case 2
        varargout{1} = dss(...
            blkdiag(sys.A, sys.Ainf), ...
            [sys.B; sys.Binf], ...
            [sys.C, sys.Cinf], ...
            sys.D, ...
            blkdiag(sys.E, sys.Einf));
end

% Assign output information.
info.N  = size(sys.A, 1);
info.Ni = size(sys.Ainf, 1);

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

if ioformat == 0
    varargout{6} = info;
else
    varargout{2} = info;
end
