function [varargout] = ml_ct_soss_flbt(varargin)
%ML_CT_SOSS_FLBT Frequency-limited balanced truncation for s.-o. systems.
%
% SYNTAX:
%   [Mr, Er, Kr, Bur, Cpr, Cvr, Dr, info] = ML_CT_SOSS_FLBT(M, E, K, Bu, Cp, Cv, D)
%   [Mr, Er, Kr, Bur, Cpr, Cvr, Dr, info] = ML_CT_SOSS_FLBT(M, E, K, Bu, Cp, Cv, D, opts)
%
%   [rom, info] = ML_CT_SOSS_FLBT(sys)
%   [rom, info] = ML_CT_SOSS_FLBT(sys, opts)
%
% DESCRIPTION:
%   This function computes the freuqency-limited balanced truncation for a
%   second-order system of the form
%
%       M*x''(t) = -K*x(t) -  E*x'(t) + Bu*u(t)                         (1)
%           y(t) = Cp*x(t) + Cv*x'(t) +  D*u(t)                         (2)
%
%   Therefore, the two generalized Lyapunov equations
%
%       A*P*E' + E*P*A' + BF*B' + B*BF' = 0,
%       A'*Q*E + E'*Q*A + CF'*C + C'*CF = 0,
%
%   where BF and CF are frequency-dependent matrices, are solved for the
%   system Gramians P and Q, with
%
%           [ I  0 ]      [ 0   I ]      [ 0  ]
%       E = [      ], A = [       ], B = [    ], C = [Cp, Cv].
%           [ 0  M ]      [ -K -E ]      [ Bu ]
%
%   As a result, a reduced-order system of the form
%
%       Mr*x''(t) = -Kr*x(t) -  Er*x'(t) + Bur*u(t)                     (3)
%            y(t) = Cpr*x(t) + Cvr*x'(t) +  Dr*u(t)                     (4)
%
%   is computed. Using the modified Gramian approach can lead to a more
%   stable reduced-order model but it is at most stable as the classical
%   second-order balanced truncation method.
%
%   Note: The method only allows asymptotically stable second-order
%         systems with invertible mass matrix.
%
% INPUT:
%   M    - matrix from (1) with dimensions n x n
%   E    - matrix from (1) with dimensions n x n
%   K    - matrix from (1) with dimensions n x n
%   Bu   - matrix from (1) with dimensions n x m
%   Cp   - matrix from (2) with dimensions p x n
%   Cv   - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure, containing the second-order system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        M        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        K        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        Bu       | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        Cp       | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        Cv       | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | BalanceType     | character array, determining the type of          |
%   |                 | second-order balancing                            |
%   |                 |   'p'   - position balancing (vel. Gram.)         |
%   |                 |   'pv'  - position-velocity balancing             |
%   |                 |   'vp'  - velocity-position balancing             |
%   |                 |   'vpv' - velocity-position balancing (vel. Gram.)|
%   |                 |   'v'   - velocity balancing                      |
%   |                 |   'fv'  - free velocity balancing                 |
%   |                 |   'so'  - second-order balancing                  |
%   |                 | (default 'so')                                    |
%   +-----------------+---------------------------------------------------+
%   | FreqRange       | nonnegative vector, frequency intervals such that |
%   |                 | [w(1), w(2)] ... [w(2k-1), w(2k)] are approximated|
%   |                 | (default [0, 1.0e+03])                            |
%   +-----------------+---------------------------------------------------+
%   | lyapdlopts      | structure, containing the optional parameters for |
%   |                 | the computation of the generalized continuous-time|
%   |                 | Lyapunov equations, see ml_lyapdl_sgn_ldl if      |
%   |                 | ModGramian = 0 and ml_lyapdl_sgn_fac if           |
%   |                 | ModGramian = 1                                    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   |                 | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | ModGramian      | {0, 1}, used to disable/enable the modified       |
%   |                 | Gramian approach                                  |
%   |                 | (default 0)                                       |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)))                     |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   |                 | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using rel. tolerance for the hsv   |
%   |                 |  'sum'       - using rel. tolerance for sum of hsv|
%   |                 | (default 'sum')                                   |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   |                 | computation of the size of the reduced-order model|
%   |                 | if 'tolerance' or 'sum' is set for                |
%   |                 | OrderComputation                                  |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Mr   - matrix of (3) with dimensions r x r
%   Er   - matrix of (3) with dimensions r x r
%   Mr   - matrix of (3) with dimensions r x r
%   Bur  - matrix of (3) with dimensions r x m
%   Cpr  - matrix of (4) with dimensions p x r
%   Cvr  - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        M        | matrix from (1) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        E        | matrix from (1) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        K        | matrix from (1) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        Bu       | matrix from (1) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        Cp       | matrix from (2) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        Cv       | matrix from (2) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Hsv             | a vector, containing the computed Hankel singular |
%   |                 | values, for second-order balancing the position   |
%   |                 | and velocity Hankel singular values are saved in  |
%   |                 | a struct                                          |
%   +-----------------+---------------------------------------------------+
%   | infoLYAPDL      | structure, containing information about the       |
%   |                 | continuous-time dual Lyapunov equations solver,   |
%   |                 | see ml_lyapdl_sgn_ldl or ml_lyapdl_sgn_fac        |
%   +-----------------+---------------------------------------------------+
%   | N               | Dimension of the reduced-order model              |
%   +-----------------+---------------------------------------------------+
%
% See also ml_ct_soss_bt, ml_ct_dss_flbt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_soss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 8);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'FreqRange')
    ml_assert_nonnegvector(opts.FreqRange, 'opts.FreqRange');
    
    assert(mod(length(opts.FreqRange), 2) == 0, ...
        'MORLAB:data', ...
        'The parameter opts.FreqRange has to be an even vector!');
    
    opts.FreqRange = sort(opts.FreqRange);
else
    opts.FreqRange = [0, 1.0e+03];
end

if ml_field_set_to_value(opts, 'lyapdlopts')
    assert(isa(opts.lyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.lyapdlopts has to be a struct!');
else
    opts.lyapdlopts = struct();
end

if ml_field_set_to_value(opts, 'Method')
    if strcmpi(opts.Method, 'bfsr')
        mor_method = @ml_ct_soss_bfsr;
    elseif strcmpi(opts.Method, 'sr')
        mor_method = @ml_ct_soss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
else
    mor_method = @ml_ct_soss_sr;
end

if ml_field_set_to_value(opts, 'ModGramian')
    ml_assert_boolean(opts.ModGramian, 'opts.ModGramian');
else
    opts.ModGramian = 0;
end

if ml_field_set_to_value(opts, 'OrderComputation')
    if strcmpi(opts.OrderComputation, 'order')
        tselect = 0;
    elseif strcmpi(opts.OrderComputation, 'tolerance')
        tselect = 1;
    elseif strcmpi(opts.OrderComputation, 'sum')
        tselect = 2;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
else
    tselect = 2;
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVE MATRIX EQUATIONS.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = size(sys.M, 1);
m = size(sys.Bu, 2);
p = max(size(sys.Cp, 1), size(sys.Cv, 1));

E = blkdiag(eye(n), sys.M);
A = [zeros(n), eye(n); -sys.K, -sys.E];
B = [zeros(n, m); sys.Bu];
C = [zeros(size(sys.Cv) - size(sys.Cp)), sys.Cp, ...
    zeros(size(sys.Cp) - size(sys.Cv)), sys.Cv];

% Computation of the full-rank factors of the proper Gramians.
if n > 0
    % Compute frequency-dependent right-hand side.
    if opts.FreqRange(1) == 0
        F      = -E \ A - 1i * opts.FreqRange(2) * eye(2*n);
        kstart = 2;
    else
        F      = eye(2*n);
        kstart = 1;
    end
    
    for k = kstart:1:length(opts.FreqRange)/2
        F = F * ((A + 1i * opts.FreqRange(2*k-1) * E) ...
            \ (A + 1i * opts.FreqRange(2*k) * E));
    end
    
    F  = (1i / pi) * logm(F);
    BF = E * real(F * (E \ B));
    CF = real(C * F);
    BQ = [zeros(m), eye(m); eye(m), zeros(m)];
    CQ = [zeros(p), eye(p); eye(p), zeros(p)];
    
    if opts.ModGramian
        % Modified Gramian approach.
        [BF, Y] = ml_compress_ldl([B, BF], BQ, eps, 'col');
        Y       = abs(diag(Y));
        BF      = BF * diag(sqrt(Y));
        
        [CF, Y] = ml_compress_ldl([C; CF], CQ, eps, 'row');
        Y       = abs(diag(Y));
        CF      = diag(sqrt(Y)) * CF;
        
        [R, L, infoLYAPDL] = ml_lyapdl_sgn_fac(A, BF, CF, E, ...
            opts.lyapdlopts);
    else
        % Classical method.
        [R, RY, L, LY, infoLYAPDL] = ml_lyapdl_sgn_ldl(A, ...
            [B, BF], BQ, [C; CF], CQ, E, opts.lyapdlopts);
        
        RY  = diag(RY);
        idx = (RY > 0);
        R   = diag(sqrt(RY(idx))) * R(idx, :);
        
        LY  = diag(LY);
        idx = (LY > 0);
        L   = diag(sqrt(LY(idx))) * L(idx, :);
    end
    
    % Assig information about Lyapunov equation solver.
    info.infoLYAPDL = infoLYAPDL;
else
    [R, L] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[sys, hsv] = mor_method(sys, R, L, tselect, opts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = sys.M;
        varargout{2} = sys.E;
        varargout{3} = sys.K;
        varargout{4} = sys.Bu;
        varargout{5} = sys.Cp;
        varargout{6} = sys.Cv;
        varargout{7} = sys.D;
        
    case 1
        varargout{1} = struct( ...
            'M' , sys.M, ...
            'E' , sys.E, ...
            'K' , sys.K, ...
            'Bu', sys.Bu, ...
            'Cp', sys.Cp, ...
            'Cv', sys.Cv, ...
            'D' , sys.D);
end

% Assign output information.
info.Hsv = hsv;
info.N   = size(sys.M, 1);

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

if ioformat == 0
    varargout{8} = info;
else
    varargout{2} = info;
end
