function [varargout] = ml_ct_ss_bst(varargin)
%ML_CT_SS_BST Balanced stochastic truncation for standard systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_BST(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_BST(A, B, C, D, opts)
%
%   [rom, info] = ML_CT_SS_BST(sys)
%   [rom, info] = ML_CT_SS_BST(sys, opts)
%
% DESCRIPTION:
%   This function computes the balanced stochastic truncation for a
%   standard system of the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefore, first the Lyapunov equation
%
%       A*P  + P*A' + B*B' = 0,
%
%   is solved for the controllability Gramian P and then, the corresponding
%   Riccati equation
%
%       A'*Q + Q*A + (C - Bw' * Q)' * inv(D*D') * (C - Bw' * Q) = 0
%
%   is solved for the Gramian Q, with
%
%       Bw = B*D' + P*C'.
%
%   As result, a reduced-order system of the form
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t)                                       (4)
%
%   is computed, such that for the original transfer function G and and the
%   r-th order transfer function Gr it holds
%
%       ||G - Gr||_{\infty} / ||G||_{\infty}
%       <= ((1 + Hsv(r+1))/(1 - Hsv(r+1)) * ...
%          * (1 + Hsv(n))/(1 - Hsv(n))) + 1,
%
%   with Hsv, a vector containing the characteristic stochastic singular
%   values of the system. If the transfer function is invertible it holds
%
%       ||inv(G)*(G - Gr)||_{\infty}
%       <= ((1 + Hsv(r+1))/(1 - Hsv(r+1)) * ...
%          * (1 + Hsv(n))/(1 - Hsv(n))) + 1.
%
%   Notes:
%     1) The equations above are defined for the case of p < m. Otherwise
%        the system is transposed, then reduced and back transposed.
%     2) In case of a rank-deficient D term, an epsilon regularization is
%        performed, which replaces the D during the computations with an
%        identity matrix scaled by a given epsilon.
%     3) For unstable systems, first an additive decomposition into the
%        stable and anti-stable parts is performed and then only the
%        stable part will be reduced.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Epsilon         | positive scalar, used in the case of a            |
%   |                 | non-full-rank D term for epsilon regularization   |
%   |                 | by multiplying with an identity matrix of         |
%   |                 | appropriate size                                  |
%   |                 | (default 1.0e-03)                                 |
%   +-----------------+---------------------------------------------------+
%   | lyapopts        | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | Lyapunov equation, see ml_lyap_sgn_fac            |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   |                 | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)) + Nu)                |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   |                 | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | pcareopts       | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | positive Riccati equation, see ml_pcare_nwt_fac   |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_signm                               |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | stabsylvopts    | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_sylv_sgn                            |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   |                 | computation of the size of the reduced-order model|
%   |                 | by an relative error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace, negative if unknown                     |
%   |                 | (default -1)                                      |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Hsv             | a vector, containing the computed characteristic  |
%   |                 | stochastic singular values                        |
%   +-----------------+---------------------------------------------------+
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | stable and anti-stable parts, see ml_ct_ss_adtf   |
%   +-----------------+---------------------------------------------------+
%   | infoLYAP        | structure, containing information about the       |
%   |                 | continuous-time Lyapunov equation sovler for the  |
%   |                 | controllability Gramian, see ml_lyap_sgn_fac      |
%   +-----------------+---------------------------------------------------+
%   | infoPCARE       | structure, containing information about the       |
%   |                 | continuous-time algebraic positive Riccati        |
%   |                 | equation for the observability Gramian, see       |
%   |                 | ml_pcare_nwt_fac                                  |
%   +-----------------+---------------------------------------------------+
%   | Ns              | Dimension of the stable part of the reduced-order |
%   |                 | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Nu              | Dimension of the anti-stable part of the reduced- |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | RelErrBound     | computed error bound for the relative error of the|
%   |                 | of the reduced-order model in H-infinity norm     |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, E. S. Quintana-Orti, G. Quintana-Orti, Efficient numerical
%   algorithms for balanced stochastic truncation, Int. J. Appl. Math.
%   Comput. Sci. 11 (5) (2001) 1123--1150.
%
% See also ml_ct_dss_bst, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'Epsilon')
    ml_assert_posscalar(opts.Epsilon, 'opts.Epsilon');
else
    opts.Epsilon = 1.0e-03;
end

if ml_field_set_to_value(opts, 'lyapopts')
    assert(isa(opts.lyapopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.lyapopts has to be a struct!');
else
    opts.lyapopts = struct();
end

if ml_field_set_to_value(opts, 'Method')
    if strcmpi(opts.Method, 'bfsr')
        mor_method = @ml_ct_ss_bfsr;
    elseif strcmpi(opts.Method, 'sr')
        mor_method = @ml_ct_ss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
else
    mor_method = @ml_ct_ss_sr;
end

if ml_field_set_to_value(opts, 'OrderComputation')
    if strcmpi(opts.OrderComputation, 'order')
        tselect = 0;
    elseif strcmpi(opts.OrderComputation, 'tolerance')
        tselect = 4;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
else
    tselect = 4;
end

if ml_field_set_to_value(opts, 'pcareopts')
    assert(isa(opts.pcareopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.pcareopts has to be a struct!');
else
    opts.pcareopts = struct();
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into the stable and anti-stable parts.
[sys, infoADTF] = ml_ct_ss_adtf(sys, opts);

% Assign information about additive decomposition.
info.infoADTF = infoADTF;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVE MATRIX EQUATIONS.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m = size(sys.B, 2);
p = size(sys.C, 1);

% Transpose system, if there are more outputs than inputs.
if m < p
    sys            = structfun(@ctranspose, sys, 'UniformOutput', 0);
    [sys.C, sys.B] = deal(sys.B, sys.C);
end

% Computation of the low-rank factors of the system Gramians.
if size(sys.A, 1) > 0
    [R, infoLYAP] = ml_lyap_sgn_fac(sys.A', sys.B', [], opts.lyapopts);
    
    % Epsilon regularization.
    if (rank(sys.D) < min(m, p)) || (norm(sys.D, 'fro') < 1.0e+02 * eps)
        D  = opts.Epsilon * eye(min(m, p));
        Bw = sys.B * opts.Epsilon * eye(min(m, p), max(m, p))' + ...
            R' * (R * sys.C');
    else
        Bw     = sys.B * sys.D' + R' * (R * sys.C');
        [~, D] = qr(sys.D', 0);
    end
    
    [L, infoPCARE] = ml_pcare_nwt_fac(sys.A - Bw * ((D' * D) \ sys.C), ...
        Bw / D, (D' \ sys.C), [], opts.pcareopts);
    
    % Assign information about equation solver.
    info.infoLYAP  = infoLYAP;
    info.infoPCARE = infoPCARE;
else
    [R, L] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[sys, hsv] = mor_method(sys, R, L, tselect, opts);

% Transpose system back, if it was transposed before.
if m < p
    sys            = structfun(@ctranspose, sys, 'UniformOutput', 0);
    [sys.C, sys.B] = deal(sys.B, sys.C);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = blkdiag(sys.A, sys.Au);
        varargout{2} = [sys.B; sys.Bu];
        varargout{3} = [sys.C,  sys.Cu];
        varargout{4} = sys.D;
        
    case 1
        varargout{1} = struct(...
            'A', blkdiag(sys.A, sys.Au), ...
            'B', [sys.B; sys.Bu], ...
            'C', [sys.C, sys.Cu], ...
            'D', sys.D);
        
    case 2
        varargout{1} = ss(...
            blkdiag(sys.A, sys.Au), ...
            [sys.B; sys.Bu], ...
            [sys.C, sys.Cu], ...
            sys.D);
end

% Assign output information.
n = size(sys.A, 1);

info.Hsv         = hsv;
info.Ns          = n;
info.Nu          = size(sys.Au, 1);
info.RelErrBound = prod((1 + hsv(n+1:end)) ./ (1 - hsv(n+1:end))) - 1;

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

if ioformat == 0
    varargout{5} = info;
else
    varargout{2} = info;
end
