function [varargout] = ml_ct_ss_hna(varargin)
%ML_CT_SS_HNA Hankel-norm approximation for standard systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_HNA(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_HNA(A, B, C, D, opts)
%
%   [rom, info] = ML_CT_SS_HNA(sys)
%   [rom, info] = ML_CT_SS_HNA(sys, opts)
%
% DESCRIPTION:
%   This function computes the Hankel-norm approximation of a standard
%   system of the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefore, first a balanced realization is computed by using the
%   balanced truncation square-root method with an appropriate tolerance
%   for the minimal realization of the given system. Then the system is
%   transformed using the formulas given in the reference below. As
%   result, a reduced-order system of the form
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t)                                       (4)
%
%   is computed, such that for the original transfer function G and and the
%   r-th order transfer function Gr it holds
%
%       ||G - Gr||_{H}       = Hsv(r+1),
%       ||G - Gr||_{\infty} <= 2 * (Hsv(r+1) + ... + Hsv(n)),
%
%   with Hsv, a vector containing the Hankel singular values of the system.
%
%   Note: For unstable systems, first an additive decomposition into the
%         stable and anti-stable parts is performed and then only the
%         stable part will be reduced. That does not change the error
%         formulas.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | hankelsignmopts | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition after the transformation of an      |
%   |                 | all-pass system, see ml_signm                     |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | hankelsylvopts  | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition after the transformation of an      |
%   |                 | all-pass system, see ml_sylv_sgn                  |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | lyapdlopts      | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time Lyapunov   |
%   |                 | equations, see ml_lyapdl_sgn_fac                  |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | MinRelTol       | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest characteristic value to determine a       |
%   |                 | minimal realization                               |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)) + Nu)                |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   |                 | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_signm                               |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | stabsylvopts    | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, see ml_sylv_sgn                            |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   |                 | computation of the size of the reduced-order model|
%   |                 | by an absolute error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace, negative if unknown                     |
%   |                 | (default -1)                                      |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   |                 | reduced-order model in H-infinity norm            |
%   +-----------------+---------------------------------------------------+
%   | Hsv             | a vector, containing the computed Hankel singular |
%   |                 | values                                            |
%   +-----------------+---------------------------------------------------+
%   | infoADTF        | structure, containing information about the       |
%   |                 | additive decomposition of the system into its     |
%   |                 | stable and anti-stable parts, see ml_ct_ss_adtf   |
%   +-----------------+---------------------------------------------------+
%   | infoHAADTF      | structure, containing information about the       |
%   |                 | additive decomposition of the Hankel-norm         |
%   |                 | transformed system, see ml_ct_ss_adtf             |
%   +-----------------+---------------------------------------------------+
%   | infoLYAPDL      | structure, containing information about the       |
%   |                 | continuous-time dual Lyapunov equations solver,   |
%   |                 | see ml_lyapdl_sgn_fac                             |
%   +-----------------+---------------------------------------------------+
%   | Ns              | Dimension of the stable part of the reduced-order |
%   |                 | model                                             |
%   +-----------------+---------------------------------------------------+
%   | Nu              | Dimension of the anti-stable part of the reduced- |
%   |                 | order model                                       |
%   +-----------------+---------------------------------------------------+
%   | Sigma           | Chosen Hankel singular value, exact approximation |
%   |                 | error in the Hankel-norm                          |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, E. S. Quintana-Orti, G. Quintana-Orti, Computing optimal
%   Hankel norm approximations of large-scale systems, in: Proc. 43rd IEEE
%   Conf. Decision Contr., Omnipress, Madison, WI, 2004, pp. 3078--3083.
%
% See also ml_ct_dss_hna, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Fill the D matrix.
if isempty(sys.D)
    sys.D = zeros(size(sys.C, 1), size(sys.B, 2));
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'hankelsignmopts')
    assert(isa(opts.hankelsignmopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.hankelsignmopts has to be a struct!');
else
    opts.hankelsignmopts = struct();
end

if ml_field_set_to_value(opts, 'hankelsylvopts')
    assert(isa(opts.hankelsylvopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.hankelsylvopts has to be a struct!');
else
    opts.hankelsylvopts = struct();
end

if ml_field_set_to_value(opts, 'lyapdlopts')
    assert(isa(opts.lyapdlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.lyapdlopts has to be a struct!');
else
    opts.lyapdlopts = struct();
end

if ml_field_set_to_value(opts, 'MinRelTol')
    ml_assert_nonnegscalar(opts.MinRelTol, 'opts.MinRelTol');
else
    opts.MinRelTol = log(size(sys.A, 1)) * eps;
end

if ml_field_set_to_value(opts, 'OrderComputation')
    if strcmpi(opts.OrderComputation, 'order')
        tselect = 0;
    elseif strcmpi(opts.OrderComputation, 'tolerance')
        tselect = 3;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
else
    tselect = 3;
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of the system into the stable and anti-stable parts.
[sys, infoADTF] = ml_ct_ss_adtf(sys, opts);

% Assign information about additive decomposition.
info.infoADTF = infoADTF;


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE MINIMAL, BALANCED REALIZATION.                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of the full-rank factors of the system Gramians.
if size(sys.A, 1) > 0
    [R, L, infoLYAPDL] = ml_lyapdl_sgn_fac(sys.A, sys.B, sys.C, [], ...
        opts.lyapdlopts);
    
    % Assign information about Lyapunov equation solver.
    info.infoLYAPDL = infoLYAPDL;
else
    [R, L] = deal([]);
end

if ml_field_set_to_value(opts, 'Tolerance')
    tmp = opts.Tolerance;
else
    tmp = [];
end
opts.Tolerance = opts.MinRelTol;
[sys, hsv]     = ml_ct_ss_sr(sys, R, L, 1, opts);
opts.Tolerance = tmp;

n  = size(sys.A, 1);
nh = ml_order(hsv, size(sys.Au, 1), 0, tselect, opts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TRANSFORMATION TO REDUCED-ORDER MODEL.                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nh < n
    sigma = hsv(nh + 1);
    
    % Get multiplicity of the chosen Hankel singular value.
    k = 1;
    
    while k < n - nh
        if abs(sigma - hsv(nh + 1 + k)) > sigma * log(n) * sqrt(eps)
            break;
        end
        
        k = k + 1;
    end
    
    % Permutation of the system.
    if nh + k < n
        p = [1:nh, nh+k+1:n, nh+1:nh+k];
        
        sys.A = sys.A(p,p);
        sys.B = sys.B(p, :);
        sys.C = sys.C(: , p);
        hsvh  = hsv(p);
    else
        hsvh = hsv(1:n);
    end
    
    % Transformation to an all-pass error system.
    nk = n - k;
    
    U = sigma * pinv(sys.C(: , nk+1:n)') * sys.B(nk+1:n, :);
    G = sparse(1:nk, 1:nk, 1 ./ (hsvh(1:nk).^2 - sigma^2));
    S = sparse(1:nk, 1:nk, hsvh(1:nk));
    A = sys.A(1:nk, 1:nk);
    B = sys.B(1:nk, :);
    C = sys.C(: , 1:nk);
    
    rom.A = G * (sigma^2 * A' + S * A * S + C' * U * B');
    rom.B = G * (S * B - C' * U);
    rom.C = C * S - U * B';
    rom.D = sys.D + U;
    
    % Additive decomposition.
    hankelopts               = struct();
    hankelopts.stabsignmopts = opts.hankelsignmopts;
    hankelopts.stabsylvopts  = opts.hankelsylvopts;
    hankelopts.UnstabDim     = nk - nh;
    
    [rom, infoHANKEL] = ml_ct_ss_adtf(rom, hankelopts);
    
    % Assign information about additive decomposition.
    info.infoHAADTF = infoHANKEL;
else
    sigma = 0;
    rom   = sys;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = blkdiag(rom.A, sys.Au);
        varargout{2} = [rom.B; sys.Bu];
        varargout{3} = [rom.C, sys.Cu];
        varargout{4} = rom.D;
        
    case 1
        varargout{1} = struct(...
            'A', blkdiag(rom.A, sys.Au), ...
            'B', [rom.B; sys.Bu], ...
            'C', [rom.C, sys.Cu], ...
            'D', rom.D);
        
    case 2
        varargout{1} = ss(...
            blkdiag(rom.A, sys.Au), ...
            [rom.B; sys.Bu], ...
            [rom.C, sys.Cu], ...
            rom.D);
end

% Assign output information.
info.AbsErrBound = 2 * sum(hsv(nh+1:end));
info.Hsv         = hsv;
info.Ns          = nh;
info.Nu          = size(sys.Au, 1);
info.Sigma       = sigma;

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

if ioformat == 0
    varargout{5} = info;
else
    varargout{2} = info;
end
