function [varargout] = ml_ct_ss_lqgbt(varargin)
%ML_CT_SS_LQGBT LQG balanced truncation for standard systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_LQGBT(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_LQGBT(A, B, C, D, opts)
%
%   [rom, info] = ML_CT_SS_LQGBT(sys)
%   [rom, info] = ML_CT_SS_LQGBT(sys, opts)
%
% DESCRIPTION:
%   This function computes the linear-quadratic Gaussian balanced
%   truncation for a standard system of the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefor, the two algebraic Riccati equations
%
%       A*P  + P*A' + B*B' - (P*C' + B*D') * inv(Rb) * (P*C' + B*D')' = 0,
%       A'*Q + Q*A  + C'*C - (B'*Q + D'*C)' * inv(Rc) * (B'*Q + D'*C) = 0,
%
%   are solved for the Gramians P and Q, with
%
%       Rb = I + D*D',
%       Rc = I + D'*D.
%
%   As result, a reduced-order system of the form
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t)                                       (4)
%
%   is computed, such that for the original transfer function G with the
%   (right) coprime factorization G = N*inv(M) and the r-th order transfer
%   function Gr with the (right) coprime factorization Gr = Nr*inv(Mr) it
%   holds
%
%       ||[N; M] - [Nr; Mr]||_{\infty} <= 2 * (Hsv(r+1)/sqrt(1
%           + Hsv(r+1)^2) + ... + Hsv(n)/sqrt(1 + Hsv(n)^2)),
%
%   with Hsv the vector containing the characteristic LQG singular values
%   of the system.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Beta            | nonnegative scalar, used as shift of the          |
%   |                 | in Bass' algorithm for better conditioning if     |
%   |                 | StabMethod == 'lyap' is chosen only used if       |
%   |                 | RiccatiSolver = 'newton'                          |
%   |                 | (default 0.1)                                     |
%   +-----------------+---------------------------------------------------+
%   | caredlopts      | structure, containing the optional parameters for |
%   |                 | the Riccati equation sign function solver, only   |
%   |                 | used if RiccatiSolver = 'sign', see ml_caredl_sgn |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | careopts        | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | Riccati equations, only used if                   |
%   |                 | RiccatiSolver = 'newton', see ml_care_nwt_fac     |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   |                 | computation of the reduced-order model            |
%   |                 |  'sr'   - square-root method                      |
%   |                 |  'bfsr' - balancing-free square-root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)) + Nu)                |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   |                 | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | RiccatiSolver   | character array, determining the solver for the   |
%   |                 | dual Riccati equations                            |
%   |                 |  'newton' - Newton iteration                      |
%   |                 |  'sign'   - dual sign function method             |
%   |                 | (default 'sign')                                  |
%   +-----------------+---------------------------------------------------+
%   | StabMethod      | character array, determining the method of        |
%   |                 | stabilization of the system, only used if         |
%   |                 | RiccatiSolver = 'newton'                          |
%   |                 |   'abe'  - partial stabilization with the         |
%   |                 |            algebraic Bernoulli equation           |
%   |                 |   'lyap' - partial stabilization with Bass'       |
%   |                 |            algorithm                              |
%   |                 | (default 'abe')                                   |
%   +-----------------+---------------------------------------------------+
%   | stabmethodopts  | structure, containing the optional parameters for |
%   |                 | the sign function based Lyapunov or Bernoulli     |
%   |                 | equation solver used for the stabilization, only  |
%   |                 | used if RiccatiSolver = 'newton', see ml_abe_sgn  |
%   |                 | or ml_lyap_sgn                                    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | stabsignmopts   | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition into stable and anti-stable system  |
%   |                 | parts, only used if RiccatiSolver = 'newton',     |
%   |                 | see ml_signm                                      |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   |                 | computation of the size of the reduced-order model|
%   |                 | by an absolute error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%   | UnstabDim       | integer, dimension of the deflating anti-stable   |
%   |                 | subspace in the control and filter Riccati        |
%   |                 | equations, only used if RiccatiSolver = 'newton', |
%   |                 | negative if unknown                               |
%   |                 | (default -1)                                      |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | AbsErrBound     | computed error bound for the absolute error of the|
%   |                 | (right) coprime factorization in H-infinity norm  |
%   +-----------------+---------------------------------------------------+
%   | Hsv             | a vector, containing the computed characteristic  |
%   |                 | LQG singular values                               |
%   +-----------------+---------------------------------------------------+
%   | infoCARE_C      | structure, containing information about the       |
%   |                 | Newton solver for the regulator Riccati equation, |
%   |                 | see ml_care_nwt_fac                               |
%   +-----------------+---------------------------------------------------+
%   | infoCARE_O      | structure, containing information about the       |
%   |                 | Newton solver for the filter Riccati equation,    |
%   |                 | see ml_care_nwt_fac                               |
%   +-----------------+---------------------------------------------------+
%   | infoCAREDL      | structure, containing information about the       |
%   |                 | sign function solver for the dual Riccati         |
%   |                 | equations, see ml_caredl_sgn                      |
%   +-----------------+---------------------------------------------------+
%   | infoPARTSTAB_C  | structure, containing information about the       |
%   |                 | partial stabilization used for the controllability|
%   |                 | Riccati equation, see ml_ct_ss_partstab           |
%   +-----------------+---------------------------------------------------+
%   | infoPARTSTAB_O  | structure, containing information about the       |
%   |                 | partial stabilization used for the observability  |
%   |                 | Riccati equation, see ml_ct_ss_partstab           |
%   +-----------------+---------------------------------------------------+
%   | N               | Dimension of the reduced-order model              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, T. Stykel, Model order reduction for differential-algebraic
%   equations: A survey, in: A. Ilchmann, T. Reis (Eds.), Surveys in
%   Differential-Algebraic Equations IV, Differential-Algebraic Equations
%   Forum, Springer International Publishing, Cham, 2017, pp. 107--160.
%   https://doi.org/10.1007/978-3-319-46618-7_3
%
% See also ml_ct_dss_lqgbt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Fill the D matrix.
if isempty(sys.D)
    sys.D = zeros(size(sys.C, 1), size(sys.B, 2));
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'caredlopts')
    assert(isa(opts.caredlopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.careopts has to be a struct!');
else
    opts.caredlopts = struct();
end

if ml_field_set_to_value(opts, 'careopts')
    assert(isa(opts.careopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.careopts has to be a struct!');
else
    opts.careopts = struct();
end

if ml_field_set_to_value(opts, 'Method')
    if strcmpi(opts.Method, 'bfsr')
        mor_method = @ml_ct_ss_bfsr;
    elseif strcmpi(opts.Method, 'sr')
        mor_method = @ml_ct_ss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
else
    mor_method = @ml_ct_ss_sr;
end

if ml_field_set_to_value(opts, 'OrderComputation')
    if strcmpi(opts.OrderComputation, 'order')
        tselect = 0;
    elseif strcmpi(opts.OrderComputation, 'tolerance')
        tselect = 5;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
else
    tselect = 5;
end

if ml_field_set_to_value(opts, 'RiccatiSolver')
    assert(strcmpi(opts.RiccatiSolver, 'newton') ...
        || strcmpi(opts.RiccatiSolver, 'sign'), ...
        'MORLAB:data', ...
        'The desired Riccati equation solver is not implemented!');
else
    opts.RiccatiSolver = 'sign';
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARTIAL STABILIZATION AND SOLVE MATRIX EQUATIONS.                       %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of matrices for Riccati equations.
Rb = eye(size(sys.C, 1)) + sys.D * sys.D';
Lb = chol(Rb);
Rc = eye(size(sys.B, 2)) + sys.D' * sys.D;
Lc = chol(Rc);

A = sys.A - sys.B * sys.D' * (Rb \ sys.C);
B = sys.B / Lc;
C = Lb' \ sys.C;

switch lower(opts.RiccatiSolver)
    case 'newton'
        % Partial stabilization.
        [K1, infoPARTSTAB_C] = ml_ct_ss_partstab(A', C', opts);
        [K2, infoPARTSTAB_O] = ml_ct_ss_partstab(A, B, opts);
        
        % Computation of the full-rank factors of the system Gramians.
        opts.careopts.K0 = K1;
        [R, infoCARE_C]  = ml_care_nwt_fac(A', C', B', [], opts.careopts);
        
        opts.careopts.K0 = K2;
        [L, infoCARE_O]  = ml_care_nwt_fac(A, B, C, [], opts.careopts);
        
        info.infoCARE_C     = infoCARE_C;
        info.infoCARE_O     = infoCARE_O;
        info.infoPARTSTAB_C = infoPARTSTAB_C;
        info.infoPARTSTAB_O = infoPARTSTAB_O;
        
    case 'sign'
        % Compute the two system Gramians.
        opts.caredlopts.EqnType = 'both';
        [X, Y, infoCAREDL] = ml_caredl_sgn(A, B, C, [], opts.caredlopts);
        
        % Computation of the full-rank factors of the system Gramians.
        [V, D] = eig(Y);
        r      = diag(D) > eps(max(diag(D)));
        R      = diag(sqrt(diag(D(r, r)))) * V(:, r)';
        
        [V, D] = eig(X);
        r      = diag(D) > eps(max(diag(D)));
        L      = diag(sqrt(diag(D(r, r)))) * V(:, r)';
        
        info.infoCAREDL = infoCAREDL;
        
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[sys, hsv] = mor_method(sys, R, L, tselect, opts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = sys.A;
        varargout{2} = sys.B;
        varargout{3} = sys.C;
        varargout{4} = sys.D;
        
    case 1
        varargout{1} = struct(...
            'A', sys.A, ...
            'B', sys.B, ...
            'C', sys.C, ...
            'D', sys.D);
        
    case 2
        varargout{1} = ss(...
            sys.A, ...
            sys.B, ...
            sys.C, ...
            sys.D);
end

% Assign and order output information.
n  = size(sys.A, 1);

info.AbsErrBound = 2 * sum(hsv(n+1:end) ./ sqrt(1 + hsv(n+1:end).^2));
info.Hsv         = hsv;
info.N           = n;

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

if ioformat == 0
    varargout{5} = info;
else
    varargout{2} = info;
end
