function [varargout] = ml_ct_ss_mt(varargin)
%ML_CT_SS_MT Modal truncation for standard systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_MT(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_MT(A, B, C, D, opts)
%
%   [rom, info] = ML_CT_SS_MT(sys)
%   [rom, info] = ML_CT_SS_MT(sys, opts)
%
% DESCRIPTION:
%   This function computes the modal truncation for a standard system of
%   the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefore, a block diagonalization of the matrix A is performed using
%   the matrix sign function and a Sylvester equation, such that
%
%            [ Ar  0 ]                        [ Cr ]
%       A2 = [       ], B2 = [ Br, B1 ], C2 = [    ], Dr = D,
%            [ 0  A1 ]                        [ C1 ]
%
%   where Ar contains all the eigenvalues of A, which have a larger real
%   part than a given alpha. As result, the reduced-order system is given
%   by
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t).                                      (4)
%
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions p x n
%   D    - matrix from (2) with dimensions p x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions p x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Alpha           | scalar, such that all finite eigenvalues with the |
%   |                 | real part smaller than Alpha are truncated        |
%   |                 | (default -1.0)                                    |
%   +-----------------+---------------------------------------------------+
%   | signmopts       | structure, containing the optional parameters for |
%   |                 | the matrix sign function used for the             |
%   |                 | decomposition of the eigenvalues, see ml_signm    |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | sylvopts        | structure, containing the optional parameters for |
%   |                 | the Sylvester equation solver used for the        |
%   |                 | decomposition of the eigenvalues, see ml_sylv_sgn |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions p x r
%   Dr   - matrix of (4) with dimensions p x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions p x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions p x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | infoSIGNM       | structure, containing information about the       |
%   |                 | matrix sign function method, see ml_signm         |
%   +-----------------+---------------------------------------------------+
%   | infoSYLV        | structure, containing information about the       |
%   |                 | Sylvester equation solver, see ml_sylv_sgn        |
%   +-----------------+---------------------------------------------------+
%   | N               | Dimension of the reduced-order model              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   P. Benner, E. S. Quintana-Orti, Model reduction based on spectral
%   projection methods, in: P. Benner, V. Mehrmann, D. Sorensen (Eds.),
%   Dimension Reduction of Large-Scale Systems, Vol. 45 of Lect. Notes
%   Comput. Sci. Eng., Springer, Berlin/Heidelberg, Germany, 2005,
%   pp. 5--45. https://doi.org/10.1007/3-540-27909-1_1
%
% See also ml_ct_dss_mt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'Alpha')
    ml_assert_scalar(opts.Alpha, 'opts.Alpha');
else
    opts.Alpha = -1.0;
end

if ml_field_set_to_value(opts, 'signmopts')
    assert(isa(opts.signmopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.signmopts has to be a struct!');
else
    opts.signmopts = struct();
end

if ml_field_set_to_value(opts, 'sylvopts')
    assert(isa(opts.sylvopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.sylvopts has to be a struct!');
else
    opts.sylvopts = struct();
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end

% Initial info structure.
info = struct();


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DECOMPOSITION OF TRANSFER FUNCTION.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Decomposition of desired and undesired eigenvalues.
n              = size(sys.A, 1);
[Z, infoSIGNM] = ml_signm(sys.A - opts.Alpha * eye(n), opts.signmopts);

% Assign information about sign function.
info.infoSIGNM = infoSIGNM;

n1 = (n + round(trace(Z))) / 2;

if (n1 > 0) && (n1 < n)
    [Q, ~, ~] = qr(eye(n) + Z);
    
    A1     = Q(: , n1+1:end)' * sys.A * Q(: , n1+1:end);
    Wa     = Q(: , 1:n1)' * sys.A * Q(: , n1+1:end);
    sys.A  = Q(: , 1:n1)' * sys.A * Q(: , 1:n1);
    
    [Y, infoSYLV] = ml_sylv_sgn(-sys.A + opts.Alpha * eye(n1), ...
        A1 - opts.Alpha * eye(n - n1), Wa, [], [], opts.sylvopts);
    
    sys.B = Q' * sys.B;
    sys.B = sys.B(1:n1, :) - Y * sys.B(n1+1:end, :);
    
    sys.C = sys.C * Q(: , 1:n1);
    
    % Assign information about Sylvester equation solver.
    info.infoSYLV = infoSYLV;
elseif n1 == 0
    [sys.A, sys.B, sys.C] = deal([]);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = sys.A;
        varargout{2} = sys.B;
        varargout{3} = sys.C;
        varargout{4} = sys.D;
        
    case 1
        varargout{1} = sys;
        
    case 2
        varargout{1} = ss(sys.A, sys.B, sys.C, sys.D);
end

% Assign output information.
info.N =  size(sys.A, 1);

[~, perm] = sort(lower(fieldnames(info)));
info      = orderfields(info, perm);

if ioformat == 0
    varargout{5} = info;
else
    varargout{2} = info;
end
