function [varargout] = ml_ct_ss_prbt(varargin)
%ML_CT_SS_PRBT Positive-real balanced truncation for standard systems.
%
% SYNTAX:
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_PRBT(A, B, C, D)
%   [Ar, Br, Cr, Dr, info] = ML_CT_SS_PRBT(A, B, C, D, opts)
%
%   [rom, info] = ML_CT_SS_PRBT(sys)
%   [rom, info] = ML_CT_SS_PRBT(sys, opts)
%
% DESCRIPTION:
%   This function computes the positive-real balanced truncation for a
%   positive-real standard system of the form
%
%       x'(t) = A*x(t) + B*u(t),                                        (1)
%        y(t) = C*x(t) + D*u(t).                                        (2)
%
%   Therefore, the two positive-real Riccati equations
%
%       A*P  + P*A' + (P*C' - B) * inv(D + D') * (P*C' - B)' = 0,
%       A'*Q + Q*A  + (B'*Q - C)' * inv(D + D') * (B'*Q - C) = 0,
%
%   are solved for the Gramians P and Q. As result, a reduced-order
%   positive-real system of the form
%
%       x'(t) = Ar*x(t) + Br*u(t),                                      (3)
%        y(t) = Cr*x(t) + Dr*u(t)                                       (4)
%
%   is computed, such that for the original transfer function G and the
%   r-th order transfer function Gr it holds
%
%       ||inv(G + D') - inv(Gr + D')||_{\infty} <= 2 * ||R||_{2}^2
%           * (Hsv(r+1) + ... + Hsv(n)),
%
%   with Hsv, a vector containing the characteristic positive-real singular
%   values of the system, and R^2 = inv(D + D').
%
%   Note: In case of a rank-deficient D + D' term, an epsilon
%         regularization is performed, which replaces the D during the
%         computations with an identity matrix scaled by a given epsilon.
%
% INPUT:
%   A    - matrix from (1) with dimensions n x n
%   B    - matrix from (1) with dimensions n x m
%   C    - matrix from (2) with dimensions m x n
%   D    - matrix from (2) with dimensions m x m
%   sys  - structure or state-space object, containing the standard
%          system's matrices:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (1) with dimensions n x n             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (1) with dimensions n x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (2) with dimensions m x n             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (2) with dimensions m x m             |
%   +-----------------+---------------------------------------------------+
%   opts - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Epsilon         | positive scalar, used in the case of a            |
%   |                 | non-full-rank D + D' term for epsilon             |
%   |                 | regularization by multiplying with an identity    |
%   |                 | matrix of appropriate size                        |
%   |                 | (default 1.0e-03)                                 |
%   +-----------------+---------------------------------------------------+
%   | Method          | character array, determining algorithm for the    |
%   |                 | computation of the reduced-order model            |
%   |                 |  'sr'   - square root method                      |
%   |                 |  'bfsr' - balancing-free square root method       |
%   |                 | (default 'sr')                                    |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | 'order' is set for OrderComputation               |
%   |                 | (default min(10,length(Hsv)))                     |
%   +-----------------+---------------------------------------------------+
%   | OrderComputation| character array, determining the method for the   |
%   |                 | computation of the size of the reduced-order model|
%   |                 |  'order'     - take explicit order                |
%   |                 |  'tolerance' - using absolute error bound         |
%   |                 | (default 'tolerance')                             |
%   +-----------------+---------------------------------------------------+
%   | pcareopts       | structure, containing the optional parameters for |
%   |                 | the computation of the continuous-time algebraic  |
%   |                 | positive Riccati equation, see ml_pcare_nwt_fac   |
%   |                 | (default struct())                                |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used for the        |
%   |                 | computation of the size of the reduced-order model|
%   |                 | by an relative error bound if 'tolerance' is set  |
%   |                 | for OrderComputation                              |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   Ar   - matrix of (3) with dimensions r x r
%   Br   - matrix of (3) with dimensions r x m
%   Cr   - matrix of (4) with dimensions m x r
%   Dr   - matrix of (4) with dimensions m x m
%   rom  - structure or state-space object, with the following entries:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   |        A        | matrix from (3) with dimensions r x r             |
%   +-----------------+---------------------------------------------------+
%   |        B        | matrix from (3) with dimensions r x m             |
%   +-----------------+---------------------------------------------------+
%   |        C        | matrix from (4) with dimensions m x r             |
%   +-----------------+---------------------------------------------------+
%   |        D        | matrix from (4) with dimensions m x m             |
%   +-----------------+---------------------------------------------------+
%   info - structure, containing the following information:
%   +-----------------+---------------------------------------------------+
%   |      ENTRY      |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Hsv             | a vector, containing the computed characteristic  |
%   |                 | positive-real singular values                     |
%   +-----------------+---------------------------------------------------+
%   | infoPCARE_C     | structure, containing information about the       |
%   |                 | continuous-time algebraic positive Riccati        |
%   |                 | equation solver for the controllability Gramian,  |
%   |                 | see ml_pcare_nwt_fac                              |
%   +-----------------+---------------------------------------------------+
%   | infoPCARE_O     | structure, containing information about the       |
%   |                 | continuous-time algebraic positive Riccati        |
%   |                 | equation solver for the observability Gramian,    |
%   |                 | see ml_pcare_nwt_fac                              |
%   +-----------------+---------------------------------------------------+
%   | InvAbsErrBound  | computed error bound for the absolute error of the|
%   |                 | inverse transfer functions in H-infinity norm     |
%   +-----------------+---------------------------------------------------+
%   | M               | matrix with dimensions m x m, polynomial part of  |
%   |                 | zeroth order and used in the error bound          |
%   +-----------------+---------------------------------------------------+
%   | N               | Dimension of the reduced-order model              |
%   +-----------------+---------------------------------------------------+
%
%
% REFERENCE:
%   S. Gugercin, A. C. Antoulas, A survey of model reduction by balanced
%   truncation and some new results, Internat. J. Control 77 (8) (2004)
%   748--766. https://doi.org/10.1080/00207170410001713448
%
% See also ml_ct_dss_prbt, ml_morlabopts.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign input to appropriate structs.
[sys, opts, ioformat] = ml_ct_ss_lti2struct(varargin{:});

if ioformat == 0
    nargoutchk(0, 5);
else
    nargoutchk(0, 2);
end

m = size(sys.B, 2);

assert(size(sys.C, 1) == m, ...
    'MORLAB:data', ...
    ['The system has to be square.\nThe matrix C must have the ' ...
    'same number of rows as columns of matrix B.']);

% Fill the D matrix.
if isempty(sys.D)
    sys.D = zeros(m);
end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'Epsilon')
    ml_assert_posscalar(opts.Epsilon, 'opts.Epsilon');
else
    opts.Epsilon = 1.0e-03;
end

if ml_field_set_to_value(opts, 'Method')
    if strcmpi(opts.Method, 'bfsr')
        mor_method = @ml_ct_ss_bfsr;
    elseif strcmpi(opts.Method, 'sr')
        mor_method = @ml_ct_ss_sr;
    else
        error('MORLAB:data', ...
            'The given computation method is not implemented!');
    end
else
    mor_method = @ml_ct_ss_sr;
end

if ml_field_set_to_value(opts, 'OrderComputation')
    if strcmpi(opts.OrderComputation, 'order')
        tselect = 0;
    elseif strcmpi(opts.OrderComputation, 'tolerance')
        tselect = 3;
    else
        error('MORLAB:data', ...
            'The desired order computation method is not implemented!');
    end
else
    tselect = 3;
end

if ml_field_set_to_value(opts, 'pcareopts')
    assert(isa(opts.pcareopts, 'struct'), ...
        'MORLAB:data', ...
        'The parameter opts.pcareopts has to be a struct!');
else
    opts.pcareopts = struct();
end

% Convert to full struct.
if any(structfun(@(x) issparse(x), sys, 'UniformOutput', 1))
    sys = structfun(@(x) full(x), sys, 'UniformOutput', 0);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SOLVE MATRIX EQUATIONS.                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Epsilon regularization.
if (rank(sys.D + sys.D') < m) || (norm(sys.D, 'fro') < 1.0e+02 * eps)
    M = opts.Epsilon * eye(m);
    J = 2 * opts.Epsilon * eye(m);
    D = sqrt(2 * opts.Epsilon) * eye(m);
else
    M = sys.D;
    J = sys.D + sys.D';
    D = chol(J);
end

% Computation of matrices for Riccati equations.
A = sys.A - sys.B * (J \ sys.C);
B = sys.B / D;
C = D' \ sys.C;

% Computation of the full-rank factors of the system Gramians.
[R, infoPCARE_C] = ml_pcare_nwt_fac(A', C', B', [], opts.pcareopts);
[L, infoPCARE_O] = ml_pcare_nwt_fac(A, B, C, [], opts.pcareopts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTE REDUCED-ORDER MODEL.                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Scaling of the tolerance.
if ml_field_set_to_value(opts, 'Tolerance')
    ml_assert_nonnegscalar(opts.Tolerance, 'opts.Tolerance');
else
    opts.Tolerance = 1.0e-02;
end

opts.Tolerance = opts.Tolerance * min(svd(J));

% Computation of the reduced-order model.
[sys, hsv] = mor_method(sys, R, L, tselect, opts);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASSIGN OUTPUT.                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Construction of the reduced-order model.
switch ioformat
    case 0
        varargout{1} = sys.A;
        varargout{2} = sys.B;
        varargout{3} = sys.C;
        varargout{4} = sys.D;
        
    case 1
        varargout{1} = sys;
        
    case 2
        varargout{1} = ss(sys.A, sys.B, sys.C, sys.D);
end

% Assign output information.
n = size(sys.A, 1);

info = struct( ...
    'Hsv'           , hsv, ...
    'infoPCARE_C'   , infoPCARE_C, ...
    'infoPCARE_O'   , infoPCARE_O, ...
    'InvAbsErrBound', 2 / min(svd(J)) * sum(hsv(n+1:end)), ...
    'M'             , M, ...
    'N'             , n);

if ioformat == 0
    varargout{5} = info;
else
    varargout{2} = info;
end
