function [sys, hsvp, hsvi] = ml_ct_dss_sr(sys, R, Rinf, L, Linf, tselect, opts)
%ML_CT_DSS_SR Square root method for descriptor systems.
%
% SYNTAX:
%   [sys, hsvp, hsvi] = ML_CT_DSS_SR(sys, R, Rinf, L, Linf, tselect)
%   [sys, hsvp, hsvi] = ML_CT_DSS_SR(sys, R, Rinf, L, Linf, tselect, opts)
%
% DESCRIPTION:
%   Computes the reduced-order matrices of a descriptor system by the
%   square root method. Therefore, transformation matrices of the form
%
%       W    = L * U * S^(-1/2),
%       Winf = Linf * Uinf * Sinf^(-1/2),
%       T    = R * V * S^(-1/2),
%       Tinf = Rinf * Vinf * Sinf^(-1/2)
%
%   are computed, where U, Uinf, S, Sinf, V, Vinf result from singular
%   value decompositions. The computation of the reduced-order model is
%   done by
%
%            [ W' * A * T     0  ]
%       Ar = [                   ],
%            [     0          I  ]
%
%            [ W' * B       ]
%       Br = [              ],
%            [ Winf' * Binf ]
%
%       Cr = [ C * T, Cinf * Tinf ],
%
%            [ I              0            ]
%       Er = [                             ].
%            [ 0       Winf' * Einf * Tinf ]
%
% INPUT:
%   sys     - structure, containing the descriptor system in the form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | E               | matrix with dimensions nf x nf                    |
%   +-----------------+---------------------------------------------------+
%   | Einf            | matrix with dimensions ninf x ninf                |
%   +-----------------+---------------------------------------------------+
%   | A               | matrix with dimensions nf x nf                    |
%   +-----------------+---------------------------------------------------+
%   | Ainf            | matrix with dimensions ninf x ninf                |
%   +-----------------+---------------------------------------------------+
%   | B               | matrix with dimensions nf x m                     |
%   +-----------------+---------------------------------------------------+
%   | Binf            | matrix with dimensions ninf x m                   |
%   +-----------------+---------------------------------------------------+
%   | C               | matrix with dimensions p x nf                     |
%   +-----------------+---------------------------------------------------+
%   | Cinf            | matrix with dimensions p x ninf                   |
%   +-----------------+---------------------------------------------------+
%   R       - Cholesky factor of the proper controllability Gramian with
%             dimensions nr1 x nf
%   Rinf    - Cholesky factor of the improper controllability Gramian with
%             dimensions nr2 x ninf
%   L       - Cholesky factor of the proper observability Gramian with
%             dimensions nl1 x nf
%   Linf    - Cholesky factor of the improper observability Gramian with
%             dimensions nl2 x ninf
%   tselect - an integer, used to determine the computation method for the
%             order of the reduced-order model
%               0 - order is directly given by user
%               1 - computed by a relative tolerance for the hsv
%               2 - computed by a relative tolerance on the sum of hsv
%               3 - computed by absolute error bound of BT
%               4 - computed by relative error bound of BST
%               5 - computed by absolute error bound of LQGBT
%               6 - computed by absolute error bound of HinfBT
%   opts    - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, scaling term from the H-infinity |
%   |                 | balanced truncation                               |
%   +-----------------+---------------------------------------------------+
%   | ImproperTrunc   | nonnegative scalar, tolerance multiplied with the |
%   |                 | largest proper Hankel singular value of the       |
%   |                 | system to truncate the improper part              |
%   |                 | (default log(n)*eps)                              |
%   +-----------------+---------------------------------------------------+
%   | Index           | nonnegative integer, index of nilpotency of the   |
%   |                 | matrix Einf used to set an upper bound on the size|
%   |                 | of the reduced improper part, if the index is     |
%   |                 | unknown Inf is set                                |
%   |                 | (default Inf)                                     |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | tselect == 1                                      |
%   |                 | (default min(10,length(hsvp))+nu+ni)              |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used in the         |
%   |                 | different error formulas                          |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   sys     - struct, containing the transformed system matrices
%   hsvp    - vector, containing the characteristic proper singular values
%   hsvi    - vector, containing the improper Hankel singular values
%
%
% REFERENCE:
%   V. Mehrmann, T. Stykel, Balanced truncation model reduction for
%   large-scale systems in descriptor form, in: P. Benner, V. Mehrmann,
%   D. C. Sorensen (Eds.), Dimension Reduction of Large-Scale Systems,
%   Vol. 45 of Lect. Notes Comput. Sci. Eng., Springer-Verlag,
%   Berlin/Heidelberg, Germany, 2005, pp. 83--115.
%   https://doi.org/10.1007/3-540-27909-1_3
%
% See also ml_ct_ss_sr, ml_ct_dss_bfsr, ml_order.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(6, 7);

if (nargin < 7) || isempty(opts)
    opts = struct();
end

% Check of input descriptor system.
ml_assert_descsys(sys);
ml_assert_infdescsys(sys);

nf   = size(sys.A, 1);
ninf = size(sys.Ainf, 1);

assert(size(R, 2) == nf, ...
    'MORLAB:data', ...
    'The matrix R must have the same number of columns as sys.A!');

assert(size(Rinf, 2) == ninf, ...
    'MORLAB:data', ...
    'The matrix Rinf must have the same number of columns as sys.Ainf!');

assert(size(L, 2) == nf, ...
    'MORLAB:data', ...
    'The matrix L must have the same number of columns as sys.A!');

assert(size(Linf, 2) == ninf, ...
    'MORLAB:data', ...
    'The matrix Linf must have the same number of columns as sys.Ainf!');

if issparse(R), R = full(R); end
if issparse(L), L = full(L); end
if issparse(Rinf), Rinf = full(Rinf); end
if issparse(Linf), Linf = full(Linf); end

% Check and assign optional parameters.
if ml_field_set_to_value(opts, 'ImproperTrunc')
    ml_assert_nonnegscalar(opts.ImproperTrunc, 'opts.ImproperTrunc');
else
    opts.ImproperTrunc = log(nf + ninf) * eps;
end

if ml_field_set_to_value(opts, 'Index')
    ml_assert_nonnegintinf(opts.Index, 'opts.Index');
else
    opts.Index = Inf;
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF CHARACTERISTIC SINGULAR VALUES.                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[U, S, V]          = svd(L * sys.E * R', 'econ');
[Uinf, Sinf, Vinf] = svd(Linf * sys.Ainf * Rinf', 'econ');

hsvp = nonzeros(diag(S));
hsvi = nonzeros(diag(Sinf));


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF REDUCED-ORDER MODEL.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Computation of the minimal size of the improper part.
if isempty(hsvi) || ...
        (isempty(hsvp)) && (hsvi(1) <= opts.ImproperTrunc) || ...
        (~isempty(hsvp)) && (hsvi(1) <= opts.ImproperTrunc * hsvp(1))
    ninf = 0;
else
    ninf = sum(hsvi >= opts.ImproperTrunc * hsvi(1));
end

% Application of rank bound on the improper part.
ninf = min([ninf, size(sys.B, 2) * opts.Index, ...
    size(sys.C, 1) * opts.Index]);

% Computation of the order of the proper part.
if ml_field_set_to_value(sys, 'Au')
    nf = ml_order(hsvp, size(sys.Au, 1), ninf, tselect, opts);
else
    nf = ml_order(hsvp, 0, ninf, tselect, opts);
end

% Computation of the transformation matrices.
S    = sparse(1:nf, 1:nf, 1 ./ sqrt(hsvp(1:nf)));
Sinf = sparse(1:ninf, 1:ninf, 1./ sqrt(hsvi(1:ninf)));

W    = S * (U(: , 1:nf)' * L);
Winf = Sinf * (Uinf(: , 1:ninf)' * Linf);

T    = (R' * V(: , 1:nf)) * S;
Tinf = (Rinf' * Vinf(: , 1:ninf)) * Sinf;

% Computation of the reduced-order matrices.
sys.A    = W * sys.A * T;
sys.Ainf = eye(ninf);
sys.B    = W * sys.B;
sys.Binf = Winf * sys.Binf;
sys.C    = sys.C * T;
sys.Cinf = sys.Cinf * Tinf;
sys.E    = eye(nf);
sys.Einf = Winf * sys.Einf * Tinf;
