function [sys, hsv] = ml_ct_soss_bfsr(sys, R,  L, tselect, opts)
%ML_CT_SOSS_BFSR Balancing-free square root method for second-order sys.
%
% SYNTAX:
%   [sys, hsv] = ML_CT_SOSS_BFSR(sys, R, L, tselect)
%   [sys, hsv] = ML_CT_SOSS_BFSR(sys, R, L, tselect, opts)
%
% DESCRIPTION:
%   Computes the reduced-order matrices of a second-order system by the
%   balancing-free square root method. The different second-order
%   balancing methods can be chosen via an optional paramter.
%
% INPUT:
%   sys     - structure, containing the second-order system in the form:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | M               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | E               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | K               | matrix with dimensions n x n                      |
%   +-----------------+---------------------------------------------------+
%   | Bu              | matrix with dimensions n x m                      |
%   +-----------------+---------------------------------------------------+
%   | Cp              | matrix with dimensions p x n, might be empty      |
%   +-----------------+---------------------------------------------------+
%   | Cv              | matrix with dimensions p x n, might be empty      |
%   +-----------------+---------------------------------------------------+
%   R       - Cholesky factor of the controllability Gramian with
%             dimensions nr x 2*n
%   L       - Cholesky factor of the observability Gramian with
%             dimensions nl x 2*n
%   tselect - an integer, used to determine the computation method for the
%             order of the reduced-order model
%               0 - order is directly given by user
%               1 - computed by a relative tolerance for the hsv
%               2 - computed by a relative tolerance on the sum of hsv
%               3 - computed by absolute error bound of BT
%               4 - computed by relative error bound of BST
%               5 - computed by absolute error bound of LQGBT
%               6 - computed by absolute error bound of HinfBT
%   opts    - structure, containing the following optional entries:
%   +-----------------+---------------------------------------------------+
%   |    PARAMETER    |                     MEANING                       |
%   +-----------------+---------------------------------------------------+
%   | BalanceType     | character array, determining the type of          |
%   |                 | second-order balancing                            |
%   |                 |   'p'   - position balancing (vel. Gram.)         |
%   |                 |   'pv'  - position-velocity balancing             |
%   |                 |   'vp'  - velocity-position balancing             |
%   |                 |   'vpv' - velocity-position balancing (vel. Gram.)|
%   |                 |   'v'   - velocity balancing                      |
%   |                 |   'fv'  - free velocity balancing                 |
%   |                 |   'so'  - second-order balancing                  |
%   |                 | (default 'so')                                    |
%   +-----------------+---------------------------------------------------+
%   | Gamma           | positive scalar, scaling term from the H-infinity |
%   |                 | balanced truncation                               |
%   +-----------------+---------------------------------------------------+
%   | Order           | positive integer, order of the resulting          |
%   |                 | reduced-order model chosen by the user if         |
%   |                 | tselect == 1                                      |
%   |                 | (default min(10,length(hsvp)))                    |
%   +-----------------+---------------------------------------------------+
%   | Tolerance       | nonnegative scalar, tolerance used in the         |
%   |                 | different error formulas                          |
%   |                 | (default 1.0e-02)                                 |
%   +-----------------+---------------------------------------------------+
%
% OUTPUT:
%   sys     - struct, containing the transformed system matrices
%   hsv     - vector, containing the characteristic singular values
%
%
% REFERENCE:
%   T. Reis, T. Stykel, Balanced truncation model reduction of second-order
%   systems, Math. Comput. Model. Dyn. Syst. 14 (5) (2008) 391--406.
%   https://doi.org/10.1080/13873950701844170
%
% See also ml_ct_ss_bfsr, ml_ct_ss_sr, ml_order.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(4, 5);

if (nargin < 5) || isempty(opts)
    opts = struct();
end

% Check of input second-order system.
ml_assert_sosys(sys);

n = size(sys.M, 1);

assert(size(R, 2) == 2 * n, ...
    'MORLAB:data', ...
    'The matrix R must have twice the number of columns as sys.M!');

assert(size(L, 2) == 2 * n, ...
    'MORLAB:data', ...
    'The matrix L must have twice the number of columns as sys.M!');

if ml_field_set_to_value(opts, 'BalanceType')
    assert(ischar(opts.BalanceType), ...
        'MORLAB:data', ...
        'The parameter opts.BalanceType has to be a string!');
else
    opts.BalanceType = 'so';
end

if issparse(R), R = full(R); end
if issparse(L), L = full(L); end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF REDUCED-ORDER MODEL.                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch lower(opts.BalanceType)
    case 'p' % Position Balancing (with additional velocity Gramian).
        % Compute position Hankel singular values.
        [~, S, V] = svd(L(:, 1:n) * R(:, 1:n)', 'econ');
        [U, ~, ~] = svd(L(:, n+1:end) * sys.M * R(:, n+1:end)', 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        r = ml_order(hsv, 0, 0, tselect, opts);
        
        % Compute the transformation matrices.
        [W, ~] = qr(L(:, n+1:end)' * U(:, 1:r), 0);
        [T, ~] = qr(R(:, 1:n)' * V(:, 1:r), 0);
        
        % Compute reduced-order model.
        sys.M  = W' * sys.M * T;
        sys.E  = W' * sys.E * T;
        sys.K  = W' * sys.K * T;
        sys.Bu = W' * sys.Bu;
        
        if ml_field_set_to_value(sys, 'Cp')
            sys.Cp = sys.Cp * T;
        end
        
        if ml_field_set_to_value(sys, 'Cv')
            sys.Cv = sys.Cv * T;
        end
        
    case 'pv' % Position-Velocity Balancing.
        % Compute position-velocity Hankel singular values.
        [U, S, V] = svd(L(:, n+1:end) * sys.M * R(:, 1:n)', 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        r = ml_order(hsv, 0, 0, tselect, opts);
        
        % Compute the transformation matrices.
        [W, ~] = qr(L(:, n+1:end)' * U(:, 1:r), 0);
        [T, ~] = qr(R(:, 1:n)' * V(:, 1:r), 0);
        
        % Compute reduced-order model.
        sys.M  = W' * sys.M * T;
        sys.E  = W' * sys.E * T;
        sys.K  = W' * sys.K * T;
        sys.Bu = W' * sys.Bu;
        
        if ml_field_set_to_value(sys, 'Cp')
            sys.Cp = sys.Cp * T;
        end
        
        if ml_field_set_to_value(sys, 'Cv')
            sys.Cv = sys.Cv * T;
        end
        
    case 'vp' % Velocity-Position Balancing.
        % Compute velocity-position Hankel singular values.
        [U, S, V] = svd(L(:, 1:n) * R(:, n+1:end)', 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        r = ml_order(hsv, 0, 0, tselect, opts);
        
        % Compute the transformation matrices.
        [W, ~] = qr(L(:, 1:n)' * U(:, 1:r), 0);
        [T, ~] = qr(R(:, n+1:end)' * V(:, 1:r), 0);
        
        % Compute reduced-order model.
        sys.M  = W' * sys.M * T;
        sys.E  = W' * sys.E * T;
        sys.K  = W' * sys.K * T;
        sys.Bu = W' * sys.Bu;
        
        if ml_field_set_to_value(sys, 'Cp')
            sys.Cp = sys.Cp * T;
        end
        
        if ml_field_set_to_value(sys, 'Cv')
            sys.Cv = sys.Cv * T;
        end
        
    case 'vpv' % Velocity-Position Balancing (with add. position Gramian).
        % Compute velocity-position Hankel singular values.
        [~, S, V] = svd(L(:, 1:n) * R(:, n+1:end)', 'econ');
        [U, ~, ~] = svd(L(:, n+1:end) * sys.M * R(:, n+1:end)', 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        r = ml_order(hsv, 0, 0, tselect, opts);
        
        % Compute the transformation matrices.
        [W, ~] = qr(L(:, n+1:end)' * U(:, 1:r), 0);
        [T, ~] = qr(R(:, n+1:end)' * V(:, 1:r), 0);
        
        % Compute reduced-order model.
        sys.M  = W' * sys.M * T;
        sys.E  = W' * sys.E * T;
        sys.K  = W' * sys.K * T;
        sys.Bu = W' * sys.Bu;
        
        if ml_field_set_to_value(sys, 'Cp')
            sys.Cp = sys.Cp * T;
        end
        
        if ml_field_set_to_value(sys, 'Cv')
            sys.Cv = sys.Cv * T;
        end
        
    case 'v' % Velocity Balancing.
        % Compute velocity Hankel singular values.
        [U, S, V] = svd(L(:, n+1:end) * sys.M * R(:, n+1:end)', 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        r = ml_order(hsv, 0, 0, tselect, opts);
        
        % Compute the transformation matrices.
        [W, ~] = qr(L(:, n+1:end)' * U(:, 1:r), 0);
        [T, ~] = qr(R(:, n+1:end)' * V(:, 1:r), 0);
        
        % Compute reduced-order model.
        sys.M  = W' * sys.M * T;
        sys.E  = W' * sys.E * T;
        sys.K  = W' * sys.K * T;
        sys.Bu = W' * sys.Bu;
        
        if ml_field_set_to_value(sys, 'Cp')
            sys.Cp = sys.Cp * T;
        end
        
        if ml_field_set_to_value(sys, 'Cv')
            sys.Cv = sys.Cv * T;
        end
        
    case 'fv' % Free Velocity Balancing.
        % Compute the position Hankel singular values.
        [U, S, V] = svd(L(:, 1:n) * R(:, 1:n)', 'econ');
        hsv       = diag(S);
        hsv       = hsv(1:min(length(hsv), n));
        
        % Computation of the order of the reduced-order system.
        r = ml_order(hsv, 0, 0, tselect, opts);
        
        % Compute the transformation matrices.
        [W, ~] = qr(L(:, 1:n)' * U(:, 1:r), 0);
        [T, ~] = qr(R(:, 1:n)' * V(:, 1:r), 0);
        
        % Compute reduced-order model.
        sys.M  = W' * sys.M * T;
        sys.E  = W' * sys.E * T;
        sys.K  = W' * sys.K * T;
        sys.Bu = W' * sys.Bu;
        
        if ml_field_set_to_value(sys, 'Cp')
            sys.Cp = sys.Cp * T;
        end
        
        if ml_field_set_to_value(sys, 'Cv')
            sys.Cv = sys.Cv * T;
        end
        
    case 'so' % Second-Order Balancing.
        % Compute position and velocity Hankel singular values.
        [Up, Sp, Vp] = svd(L(:, 1:n) * R(:, 1:n)', 'econ');
        hsvp         = diag(Sp);
        hsvp         = hsvp(1:min(length(hsvp), n));
        [Uv, Sv, Vv] = svd(L(:, n+1:end) * sys.M * R(:, n+1:end)', 'econ');
        hsvv         = diag(Sv);
        hsvv         = hsvv(1:min(length(hsvv), n));
        
        % Computation of the order of the reduced-order system.
        r = min(ml_order(hsvp, 0, 0, tselect, opts), ...
            ml_order(hsvv, 0, 0, tselect, opts));
        
        % Compute the transformation matrices.
        [Wp, ~] = qr(L(:, 1:n)' * Up(:, 1:r), 0);
        [Wv, ~] = qr(L(:, n+1:end)' * Uv(:, 1:r), 0);
        [Tp, ~] = qr(R(:, 1:n)' * Vp(:, 1:r), 0);
        [Tv, ~] = qr(R(:, n+1:end)' * Vv(:, 1:r), 0);
        
        S = (Wp' * Tp) \ (Wp' * Tv);
        
        % Compute reduced-order model.
        sys.M  = S * ((Wv' * sys.M * Tv) / S);
        sys.E  = S * ((Wv' * sys.E * Tv) / S);
        sys.K  = S * (Wv' * sys.K * Tp);
        sys.Bu = S * (Wv' * sys.Bu);
        
        if ml_field_set_to_value(sys, 'Cp')
            sys.Cp = sys.Cp * Tp;
        end
        
        if ml_field_set_to_value(sys, 'Cv')
            sys.Cv = (sys.Cv * Tv) / S;
        end
        
        hsv = struct('Hsvp', hsvp, 'Hsvv', hsvv);
        
    otherwise
        error('MORLAB:data', ...
            'The required balancing method is not implemented!');
end
