function [opts] = ml_morlabopts(fname)
%ML_MORLABOPTS Option struct generator for MORLAB functions.
%
% SYNTAX:
%   ML_MORLABOPTS(fname)
%   opts = ML_MORLABOPTS(fname)
%
% DESCRIPTION:
%   This function can be used to generate an option structure for the
%   MORLAB function specified by fname. The structure is filled with empty
%   arrays such that the default values of the MORLAB function are called.
%   For the choice of the default values, call this function without an
%   output argument or see the documentation of the corresponding function.
%
% INPUT:
%   fname - character array, the name of the function for which the option
%           struct is requested
%
% OUTPUT:
%   opts - struct, the desired option structure for a MORLAB function
%
% See also morlab.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUT.                                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(1, 1);

assert(ischar(fname), ...
    'MORLAB:data', ...
    'The given function name has to be a character array.');

% Check if hyperlinks can be used.
hasHyperlink = usejava('desktop');

% Simplify case checks by restricting to lower case letters.
fname = lower(fname);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHOOSE FUNCTION.                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Assign option structs.
switch fname
    case 'ml_addpath' % On-top functions.
        if nargout == 0
            fprintf(1, [' The requested function doesn''t use an ' ...
                'option struct!']);
            fprintf(1, '\n');
        else
            opts = struct([]);
        end
        
    case { ... % Check functions.
            'ml_assert_boolean', ...
            'ml_assert_descsys', ...
            'ml_assert_infdescsys', ...
            'ml_assert_integer', ...
            'ml_assert_nonnegint', ...
            'ml_assert_nonnegintinf', ...
            'ml_assert_nonnegscalar', ...
            'ml_assert_nonnegvector', ...
            'ml_assert_posinteger', ...
            'ml_assert_posscalar', ...
            'ml_assert_scalar', ...
            'ml_assert_sosys', ...
            'ml_assert_stdsys', ...
            'ml_field_set_to_value'}
        if nargout == 0
            fprintf(1, [' The requested function doesn''t use an ' ...
                'option struct!']);
            fprintf(1, '\n');
        else
            opts = struct([]);
        end
        
    case { ... % Equation solver routines.
            'ml_abe_sgn', ...
            'ml_lyap_sgn', ...
            'ml_sylv_sgn'}
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+01*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol' , [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case 'ml_care_nwt_fac'
        if nargout == 0
            fprintf(1, ['   AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['     Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['       K0: [ matrix, s.t. s*E - (A - BK0) ' ...
                'Hurwitz | {zeros(m, n)} ]\n']);
            if hasHyperlink
                fprintf(1, [' lyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">' ...
                    'ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, [' lyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['  MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['   RelTol: [ nonnegative scalar | ' ...
                '{1.0e+02*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol'  , [], ...
                'Info'    , [], ...
                'K0'      , [], ...
                'lyapopts', ml_morlabopts('ml_lyap_sgn_fac'), ...
                'MaxIter' , [], ...
                'RelTol'  , []);
        end
        
    case 'ml_caredl_sgn'
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, [' CompTol: [ nonnegative scalar | ' ...
                '{1.0e-02*sqrt(n*eps)} ]\n']);
            fprintf(1, [' EqnType: [ primal | dual | ' ...
                '{both} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+01*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol' , [], ...
                'CompTol', [], ...
                'EqnType', [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case 'ml_gdlyap_smith_fac'
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['   Index: [ integer | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+01*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol' , [], ...
                'Index'  , [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case 'ml_icare_ric_fac'
        if nargout == 0
            fprintf(1, ['   AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['  CompTol: [ nonnegative scalar | ' ...
                '{1.0e-02*sqrt(n*eps)} ]\n']);
            fprintf(1, ['     Info: [ 1 | ' ...
                '{0} ]\n']);
            if hasHyperlink
                fprintf(1, [' careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, [' careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['  MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['   RelTol: [ nonnegative scalar | ' ...
                '{1.0e+02*n*eps} ]\n']);
            fprintf(1, [' StoreLQG: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['       Z0: [ matrix (nz x n) | ' ...
                '{[]} ]\n']);
            
        else
            opts = struct( ...
                'AbsTol'  , [], ...
                'CompTol' , [], ...
                'Info'    , [], ...
                'careopts', ml_morlabopts('ml_care_nwt_fac'), ...
                'MaxIter' , [], ...
                'RelTol'  , [], ...
                'StoreLQG', [], ...
                'Z0'      , []);
            
        end
        
    case { ...
            'ml_lyap_sgn_fac', ...
            'ml_lyapdl_sgn_fac'}
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, [' CompTol: [ nonnegative scalar | ' ...
                '{1.0e-02*sqrt(n*eps)} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+01*n*eps} ]\n']);
        else
            opts = struct( ...
                'AbsTol' , [], ...
                'CompTol', [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case { ...
            'ml_lyap_sgn_ldl', ...
            'ml_lyapdl_sgn_ldl'}
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, [' CompTol: [ nonnegative scalar | ' ...
                '{sqrt(n)*eps} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+01*n*eps} ]\n']);
        else
            opts = struct( ...
                'AbsTol' , [], ...
                'CompTol', [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case 'ml_pcare_nwt_fac'
        if nargout == 0
            fprintf(1, ['   AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['  CompTol: [ nonnegative scalar | ' ...
                '{1.0e-02*sqrt(n*eps)} ]\n']);
            fprintf(1, ['     Info: [ 1 | ' ...
                '{0} ]\n']);
            if hasHyperlink
                fprintf(1, [' lyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">' ...
                    'ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, [' lyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['  MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['   RelTol: [ nonnegative scalar | ' ...
                '{1.0e+02*n*eps} ]\n']);
        else
            opts = struct( ...
                'AbsTol'  , [], ...
                'CompTol' , [], ...
                'Info'    , [], ...
                'lyapopts', ml_morlabopts('ml_lyap_sgn_fac'), ...
                'MaxIter' , [], ...
                'RelTol'  , []);
        end
        
    case 'ml_ct_ss_brbt' % Standard model order reduction routines.
        if nargout == 0
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_ss_bst'
        if nargout == 0
            fprintf(1, ['          Epsilon: [ positive scalar | ' ...
                '{1.0e-03} ]\n']);
            if hasHyperlink
                fprintf(1, ['         lyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">' ...
                    'ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['         lyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct( ...
                'Epsilon'         , [], ...
                'lyapopts'        , ml_morlabopts('ml_lyap_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'    , ml_morlabopts('ml_sylv_sgn'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_ct_ss_bt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct(...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'    , ml_morlabopts('ml_sylv_sgn'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_ct_ss_flbt'
        if nargout == 0
            fprintf(1, ['        FreqRange: [ nonnegative vector | ' ...
                '{[0, 1.0e+03]} ]\n']);
            if hasHyperlink
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or ' ...
                    '<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ' ...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['       ModGramian: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '''tolerance'' | {''sum''} ]\n']);
            if hasHyperlink
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct( ...
                'FreqRange'       , [], ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'          , [], ...
                'ModGramian'      , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'    , ml_morlabopts('ml_sylv_sgn'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_ct_ss_hinfbt'
        if nargout == 0
            fprintf(1, ['             Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            if hasHyperlink
                fprintf(1, ['       caredlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_caredl_sgn">' ...
                    'ml_morlabopts(''ml_caredl_sgn'')</a>} ]\n']);
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['         caredlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_caredl_sgn'')} ]\n']);
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['            Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            fprintf(1, ['    RiccatiSolver: [ ''newton'' | ' ...
                '{''sign''} ]\n']);
            fprintf(1, ['       StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
            else
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            temp = ml_morlabopts('ml_care_nwt_fac');
            temp = rmfield(temp, 'K0');
            
            opts = struct(...
                'Beta'            , [], ...
                'caredlopts'      , ml_morlabopts('ml_caredl_sgn'), ...
                'careopts'        , temp, ...
                'Gamma'           , [], ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'   , [], ...
                'StabMethod'      , [], ...
                'stabmethodopts'  , ml_morlabopts('ml_abe_sgn'), ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_ct_ss_hna'
        if nargout == 0
            if hasHyperlink
                fprintf(1, ['  hankelsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">'...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['   hankelsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['  hankelsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['   hankelsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['        MinRelTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct(...
                'hankelsignmopts' , ml_morlabopts('ml_signm'),...
                'hankelsylvopts'  , ml_morlabopts('ml_sylv_sgn'), ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'MinRelTol'       , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'    , ml_morlabopts('ml_sylv_sgn'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_ct_ss_lqgbt'
        if nargout == 0
            fprintf(1, ['             Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            if hasHyperlink
                fprintf(1, ['       caredlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_caredl_sgn">' ...
                    'ml_morlabopts(''ml_caredl_sgn'')</a>} ]\n']);
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['         caredlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_caredl_sgn'')} ]\n']);
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            fprintf(1, ['    RiccatiSolver: [ ''newton'' | ' ...
                '{''sign''} ]\n']);
            fprintf(1, ['       StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
            else
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            temp = ml_morlabopts('ml_care_nwt_fac');
            temp = rmfield(temp, 'K0');
            
            opts = struct(...
                'Beta'            , [], ...
                'caredlopts'      , ml_morlabopts('ml_caredl_sgn'), ...
                'careopts'        , temp, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'   , [], ...
                'StabMethod'      , [], ...
                'stabmethodopts'  , ml_morlabopts('ml_abe_sgn'), ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_ct_ss_mt'
        if nargout == 0
            fprintf(1, ['     Alpha: [ real scalar | ' ...
                '{-1.0} ]\n']);
            if hasHyperlink
                fprintf(1, [' signmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['  sylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, [' signmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['  sylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
        else
            opts = struct(...
                'Alpha'     , [], ...
                'signmopts' , ml_morlabopts('ml_signm'), ...
                'sylvopts'  , ml_morlabopts('ml_sylv_sgn'));
        end
        
    case 'ml_ct_ss_prbt'
        if nargout == 0
            fprintf(1, ['          Epsilon: [ positive scalar | ' ...
                '{1.0e-03} ]\n']);
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv))} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Epsilon'         , [], ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_ss_tlbt'
        if nargout == 0
            if hasHyperlink
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or ' ...
                    '<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ' ...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['       ModGramian: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv)) + Nu} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '''tolerance'' | {''sum''} ]\n']);
            if hasHyperlink
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['    stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['     stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['        TimeRange: [ nonnegative vector | ' ...
                '{[0, 10]} ]\n']);
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
            fprintf(1, ['        UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct( ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'          , [], ...
                'ModGramian'      , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabsignmopts'   , ml_morlabopts('ml_signm'), ...
                'stabsylvopts'    , ml_morlabopts('ml_sylv_sgn'), ...
                'TimeRange'       , [], ...
                'Tolerance'       , [], ...
                'UnstabDim'       , []);
        end
        
    case 'ml_ct_dss_brbt' % Descriptor model order reduction routines.
        if nargout == 0
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_ct_dss_adtf');
            
            opts = struct( ...
                'DecompEig'       , [], ...
                'DecompTol'       , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_dss_bst'
        if nargout == 0
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['          Epsilon: [ positive scalar | ' ...
                '{1.0e-03} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['         lyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyap_sgn_fac">' ...
                    'ml_morlabopts(''ml_lyap_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['         lyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyap_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_ct_dss_adtf');
            
            opts = struct(...
                'DecompEig'       , [], ...
                'DecompTol'       , temp.DecompTol, ...
                'Epsilon'         , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'lyapopts'        , ml_morlabopts('ml_lyap_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'stabdecopts'     , temp.stabdecopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_dss_bt'
        if nargout == 0
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_ct_dss_adtf');
            
            opts = struct( ...
                'DecompEig'       , [], ...
                'DecompTol'       , temp.DecompTol, ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabdecopts'     , temp.stabdecopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_dss_flbt'
        if nargout == 0
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['        FreqRange: [ nonnegative vector | ' ...
                '{[0, 1.0e+03]} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or ' ...
                    '<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['       ModGramian: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_ct_dss_adtf');
            
            opts = struct( ...
                'DecompEig'       , [], ...
                'DecompTol'       , temp.DecompTol, ...
                'FreqRange'       , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'          , [], ...
                'ModGramian'      , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabdecopts'     , temp.stabdecopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_dss_hinfbt'
        if nargout == 0
            fprintf(1, ['             Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            if hasHyperlink
                fprintf(1, ['       caredlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_caredl_sgn">' ...
                    'ml_morlabopts(''ml_caredl_sgn'')</a>} ]\n']);
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       caredlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_caredl_sgn">' ...
                    'ml_morlabopts(''ml_caredl_sgn'')</a>} ]\n']);
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            fprintf(1, ['    RiccatiSolver: [ ''newton'' | ' ...
                '{''sign''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['       StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp1 = ml_morlabopts('ml_ct_dss_adtf');
            temp2 = ml_morlabopts('ml_ct_dss_partstab');
            temp3 = ml_morlabopts('ml_care_nwt_fac');
            temp3 = rmfield(temp3, 'K0');
            
            opts = struct(...
                'Beta'            , temp2.Beta, ...
                'caredlopts'      , ml_morlabopts('ml_caredl_sgn'), ...
                'careopts'        , temp3, ...
                'DecompEig'       , [], ...
                'DecompTol'       , temp1.DecompTol, ...
                'Gamma'           , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp1.infdecopts, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'   , [], ...
                'stabdecopts'     , temp1.stabdecopts, ...
                'StabMethod'      , temp2.StabMethod, ...
                'stabmethodopts'  , temp2.stabmethodopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_dss_hna'
        if nargout == 0
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
                fprintf(1, ['    hankeldecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
                fprintf(1, ['    hankeldecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['        MinRelTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp1 = ml_morlabopts('ml_ct_dss_adtf');
            temp2 = rmfield(temp1.stabdecopts, 'Dimension');
            
            opts = struct( ...
                'DecompEig'       , [], ...
                'DecompTol'       , temp1.DecompTol, ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'hankeldecopts'   , temp2, ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp1.infdecopts, ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'MinRelTol'       , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabdecopts'     , temp1.stabdecopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_dss_lqgbt'
        if nargout == 0
            fprintf(1, ['             Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            if hasHyperlink
                fprintf(1, ['       caredlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_caredl_sgn">' ...
                    'ml_morlabopts(''ml_caredl_sgn'')</a>} ]\n']);
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_care_nwt_fac">' ...
                    'ml_morlabopts(''ml_care_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       caredlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_caredl_sgn">' ...
                    'ml_morlabopts(''ml_caredl_sgn'')</a>} ]\n']);
                fprintf(1, ['         careopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_care_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            fprintf(1, ['    RiccatiSolver: [ ''newton'' | ' ...
                '{''sign''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['       StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
            else
                fprintf(1, ['   stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp1 = ml_morlabopts('ml_ct_dss_adtf');
            temp2 = ml_morlabopts('ml_ct_dss_partstab');
            temp3 = ml_morlabopts('ml_care_nwt_fac');
            temp3 = rmfield(temp3, 'K0');
            
            opts = struct(...
                'Beta'            , temp2.Beta, ...
                'caredlopts'      , ml_morlabopts('ml_caredl_sgn'), ...
                'careopts'        , temp3, ...
                'DecompEig'       , [], ...
                'DecompTol'       , temp1.DecompTol, ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp1.infdecopts, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'RiccatiSolver'   , [], ...
                'stabdecopts'     , temp1.stabdecopts, ...
                'StabMethod'      , temp2.StabMethod, ...
                'stabmethodopts'  , temp2.stabmethodopts, ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_dss_mt'
        if nargout == 0
            fprintf(1, ['      Alpha: [ real scalar | ' ...
                '{-1.0} ]\n']);
            fprintf(1, ['  DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['  DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['   diskopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>} ]\n']);
                fprintf(1, [' infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['   diskopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk'')} ]\n']);
                fprintf(1, [' infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['    RankTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
        else
            temp = ml_morlabopts('ml_ct_dss_adtf');
            
            opts = struct( ...
                'Alpha'     , [], ...
                'DecompEig' , [], ...
                'DecompTol' , temp.DecompTol, ...
                'diskopts'  , ml_morlabopts('ml_disk'), ...
                'infdecopts', temp.infdecopts, ...
                'RankTol'   , []);
        end
        
    case 'ml_ct_dss_prbt'
        if nargout == 0
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['          Epsilon: [ positive scalar | ' ...
                '{1.0e-03} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_pcare_nwt_fac">' ...
                    'ml_morlabopts(''ml_pcare_nwt_fac'')</a>} ]\n']);
            else
                fprintf(1, ['        pcareopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_pcare_nwt_fac'')} ]\n']);
            end
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_ct_dss_adtf');
            
            opts = struct( ...
                'DecompEig'       , [], ...
                'DecompTol'       , temp.DecompTol, ...
                'Epsilon'         , [], ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'pcareopts'       , ml_morlabopts('ml_pcare_nwt_fac'), ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_dss_tlbt'
        if nargout == 0
            fprintf(1, ['        DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['        DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ' ...
                    'ml_gdlyap_smith_fac">' ...
                    'ml_morlabopts(''ml_gdlyap_smith_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       gdlyapopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_gdlyap_smith_fac'')} ]\n']);
            end
            fprintf(1, ['    ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['            Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            if hasHyperlink
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or ' ...
                    '<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['       ModGramian: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '{''tolerance''} ]\n']);
            if hasHyperlink
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['      stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['        TimeRange: [ nonnegative vector | ' ...
                '{[0, 10]} ]\n']);
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            temp = ml_morlabopts('ml_ct_dss_adtf');
            
            opts = struct( ...
                'DecompEig'       , [], ...
                'DecompTol'       , temp.DecompTol, ...
                'gdlyapopts'      , ...
                ml_morlabopts('ml_gdlyap_smith_fac'), ...
                'ImproperTrunc'   , [], ...
                'Index'           , [], ...
                'infdecopts'      , temp.infdecopts, ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'          , [], ...
                'ModGramian'      , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'stabdecopts'     , temp.stabdecopts, ...
                'TimeRange'       , [], ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_soss_bt' % Second-order model reduction routines.
        if nargout == 0
            fprintf(1, ['      BalanceType: [ ''p'' | ' ...
                '''pv'' | ''vp'' | ''vpv'' | ''v'' | ''fv'' | ' ...
                '{''so''} ]\n']);
            if hasHyperlink
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv))} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '''tolerance'' | {''sum''} ]\n']);
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct( ...
                'BalanceType'     , [], ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_fac'), ...
                'Method'          , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_soss_flbt'
        if nargout == 0
            fprintf(1, ['      BalanceType: [ ''p'' | ' ...
                '''pv'' | ''vp'' | ''vpv'' | ''v'' | ''fv'' | ' ...
                '{''so''} ]\n']);
            fprintf(1, ['        FreqRange: [ nonnegative vector | ' ...
                '{[0, 1.0e+03]} ]\n']);
            if hasHyperlink
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or ' ...
                    '<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ' ...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['       ModGramian: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv))} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '''tolerance'' | {''sum''} ]\n']);
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct( ...
                'FreqRange'       , [], ...
                'BalanceType'     , [], ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'          , [], ...
                'ModGramian'      , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_soss_tlbt'
        if nargout == 0
            fprintf(1, ['      BalanceType: [ ''p'' | ' ...
                '''pv'' | ''vp'' | ''vpv'' | ''v'' | ''fv'' | ' ...
                '{''so''} ]\n']);
            if hasHyperlink
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_lyapdl_sgn_ldl">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_ldl'')</a> or ' ...
                    '<a href="matlab:ml_morlabopts ml_lyapdl_sgn_fac">'...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')</a>} ]\n']);
            else
                fprintf(1, ['       lyapdlopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_lyapdl_sgn_ldl'') or ' ...
                    'ml_morlabopts(''ml_lyapdl_sgn_fac'')} ]\n']);
            end
            fprintf(1, ['           Method: [ ''bfsr'' | ' ...
                '{''sr''} ]\n']);
            fprintf(1, ['       ModGramian: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, ['            Order: [ positive integer | ' ...
                '{min(10,length(Hsv))} ]\n']);
            fprintf(1, [' OrderComputation: [ ''order'' | ' ...
                '''tolerance'' | {''sum''} ]\n']);
            fprintf(1, ['        TimeRange: [ nonnegative vector | ' ...
                '{[0, 10]} ]\n']);
            fprintf(1, ['        Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct( ...
                'BalanceType'     , [], ...
                'lyapdlopts'      , ml_morlabopts('ml_lyapdl_sgn_ldl'), ...
                'Method'          , [], ...
                'ModGramian'      , [], ...
                'Order'           , [], ...
                'OrderComputation', [], ...
                'TimeRange'       , [], ...
                'Tolerance'       , []);
        end
        
    case 'ml_ct_ss_adtf' % Subroutines.
        if nargout == 0
            if hasHyperlink
                fprintf(1, [' stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
                fprintf(1, ['  stabsylvopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_sylv_sgn">' ...
                    'ml_morlabopts(''ml_sylv_sgn'')</a>} ]\n']);
            else
                fprintf(1, [' stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
                fprintf(1, ['  stabsylvopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_sylv_sgn'')} ]\n']);
            end
            fprintf(1, ['     UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct( ...
                'stabsignmopts', ml_morlabopts('ml_signm'), ...
                'stabsylvopts' , ml_morlabopts('ml_sylv_sgn'), ...
                'UnstabDim'    , []);
        end
        
    case {'ml_ct_ss_bfsr', 'ml_ct_ss_sr'}
        if nargout == 0
            fprintf(1, ['     Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['     Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu} ]\n']);
            fprintf(1, [' Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct( ...
                'Gamma'    , [], ...
                'Order'    , [], ...
                'Tolerance', []);
        end
        
    case { ...
            'ml_ct_dss_lti2struct', ...
            'ml_ct_ss_lti2struct', ...
            'ml_ct_soss_lti2struct'}
        if nargout == 0
            fprintf(1, [' The requested function is an input parser ' ...
                'and can\n take arbitrary option structs!\n']);
        else
            opts = struct([]);
        end
        
    case {'ml_disk', 'ml_signm'}
        if nargout == 0
            fprintf(1, ['  AbsTol: [ nonnegative scalar | ' ...
                '{0} ]\n']);
            fprintf(1, ['    Info: [ 1 | ' ...
                '{0} ]\n']);
            fprintf(1, [' MaxIter: [ positive integer | ' ...
                '{100} ]\n']);
            fprintf(1, ['  RelTol: [ nonnegative scalar | ' ...
                '{1.0e+02*n*eps} ]\n']);
        else
            opts = struct(...
                'AbsTol' , [], ...
                'Info'   , [], ...
                'MaxIter', [], ...
                'RelTol' , []);
        end
        
    case 'ml_ct_dss_adtf'
        if nargout == 0
            fprintf(1, ['   DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['   DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['  infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, [' stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['  infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, [' stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
        else
            temp           = ml_morlabopts('ml_disk');
            temp.Dimension = [];
            temp.RankTol   = [];
            temp           = orderfields(temp);
            
            opts = struct( ...
                'DecompEig'  , [], ...
                'DecompTol'  , [], ...
                'infdecopts' , temp, ...
                'stabdecopts', temp);
        end
        
    case {'ml_ct_dss_bfsr', 'ml_ct_dss_sr'}
        if nargout == 0
            fprintf(1, ['         Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, [' ImproperTrunc: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            fprintf(1, ['         Index: [ nonnegative integer | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['         Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu + Ni} ]\n']);
            fprintf(1, ['     Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Gamma'        , [], ...
                'ImproperTrunc', [], ...
                'Index'        , [], ...
                'Order'        , [], ...
                'Tolerance'    , []);
        end
        
    case {'ml_ct_soss_bfsr', 'ml_ct_soss_sr'}
        if nargout == 0
            fprintf(1, [' BalanceType: [ ''p'' | ' ...
                '''pv'' | ''vp'' | ''vpv'' | ''v'' | ''fv'' | ' ...
                '{''so''} ]\n']);
            fprintf(1, ['       Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['       Order: [ positive integer | ' ...
                '{min(10,length(Hsvp))} ]\n']);
            fprintf(1, ['   Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct( ...
                'BalanceType', [], ...
                'Gamma'      , [], ...
                'Order'      , [], ...
                'Tolerance'  , []);
        end
        
    case 'ml_getqz'
        if nargout == 0
            fprintf(1, [' Dimension: [ integer | ' ...
                '{-1} ]\n']);
            fprintf(1, ['   RankTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
        else
            opts = struct(...
                'Dimension', [], ...
                'RankTol'  , []);
        end
        
    case 'ml_ct_dss_partstab'
        if nargout == 0
            fprintf(1, ['           Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            fprintf(1, ['      DecompEig: [ positive scalar | ' ...
                '{[]} ]\n']);
            fprintf(1, ['      DecompTol: [ nonnegative scalar | ' ...
                '{log(n)*eps} ]\n']);
            if hasHyperlink
                fprintf(1, ['     infdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
                fprintf(1, ['    stabdecopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_disk">' ...
                    'ml_morlabopts(''ml_disk'')</a>, ' ...
                    '<a href="matlab:ml_morlabopts ml_getqz">' ...
                    'ml_morlabopts(''ml_getqz'')</a>} ]\n']);
            else
                fprintf(1, ['     infdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
                fprintf(1, ['    stabdecopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_disk''), ' ...
                    'ml_morlabopts(''ml_getqz'')} ]\n']);
            end
            fprintf(1, ['     StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, [' stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
            else
                fprintf(1, [' stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
            end
        else
            temp = ml_morlabopts('ml_ct_dss_adtf');
            
            opts = struct(...
                'Beta'          , [], ...
                'DecompEig'     , [], ...
                'DecompTol'     , temp.DecompTol, ...
                'infdecopts'    , temp.infdecopts, ...
                'stabdecopts'   , temp.stabdecopts, ...
                'StabMethod'    , [], ...
                'stabmethodopts', ml_morlabopts('ml_abe_sgn'));
        end
        
    case 'ml_order'
        if nargout == 0
            fprintf(1, ['     Gamma: [ positive scalar | ' ...
                '{Inf} ]\n']);
            fprintf(1, ['     Order: [ positive integer | ' ...
                '{min(10,length(Hsvp)) + Nu [+ Ni]} ]\n']);
            fprintf(1, [' Tolerance: [ nonnegative scalar | ' ...
                '{1.0e-02} ]\n']);
        else
            opts = struct(...
                'Gamma'    , [], ...
                'Order'    , [], ...
                'Tolerance', []);
        end
        
    case 'ml_ct_ss_partstab'
        if nargout == 0
            fprintf(1, ['           Beta: [ nonnegative scalar | ' ...
                '{0.1} ]\n']);
            fprintf(1, ['     StabMethod: [ ''lyap'' | ' ...
                '{''abe''} ]\n']);
            if hasHyperlink
                fprintf(1, [' stabmethodopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_abe_sgn">' ...
                    'ml_morlabopts(''ml_abe_sgn'')</a>} ]\n']);
                fprintf(1, ['  stabsignmopts: [ 1x1 struct | ' ...
                    '{<a href="matlab:ml_morlabopts ml_signm">' ...
                    'ml_morlabopts(''ml_signm'')</a>} ]\n']);
            else
                fprintf(1, [' stabmethodopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_abe_sgn'')} ]\n']);
                fprintf(1, ['  stabsignmopts: [ 1x1 struct | ' ...
                    '{ml_morlabopts(''ml_signm'')} ]\n']);
            end
            fprintf(1, ['      UnstabDim: [ integer | ' ...
                '{-1} ]\n']);
        else
            opts = struct(...
                'Beta'          , [], ...
                'StabMethod'    , [], ...
                'stabmethodopts', ml_morlabopts('ml_lyap_sgn'), ...
                'stabsignmopts' , ml_morlabopts('ml_signm'), ...
                'UnstabDim'     , []);
        end
        
    case {...
            'ml_compress_fac', ...
            'ml_compress_ldl', ...
            'ml_morlabopts', ...
            'ml_sigmaplot'}
        if nargout == 0
            fprintf(1, [' The requested function doesn''t use ' ...
                'an option struct!\n']);
        else
            opts = struct([]);
        end
        
    case { ... % Demo scripts
            'morlab_demo_ct_dss_brbt', ...
            'morlab_demo_ct_dss_bst', ...
            'morlab_demo_ct_dss_bt', ...
            'morlab_demo_ct_dss_flbt', ...
            'morlab_demo_ct_dss_hinfbt', ...
            'morlab_demo_ct_dss_hna', ...
            'morlab_demo_ct_dss_lqgbt', ...
            'morlab_demo_ct_dss_mt', ...
            'morlab_demo_ct_dss_prbt', ...
            'morlab_demo_ct_dss_tlbt', ...
            'morlab_demo_ct_soss_bt', ...
            'morlab_demo_ct_soss_flbt', ...
            'morlab_demo_ct_soss_tlbt', ...
            'morlab_demo_ct_ss_brbt', ...
            'morlab_demo_ct_ss_bst', ...
            'morlab_demo_ct_ss_bt', ...
            'morlab_demo_ct_ss_flbt', ...
            'morlab_demo_ct_ss_hinfbt', ...
            'morlab_demo_ct_ss_hna', ...
            'morlab_demo_ct_ss_lqgbt', ...
            'morlab_demo_ct_ss_mt', ...
            'morlab_demo_ct_ss_prbt', ...
            'morlab_demo_ct_ss_tlbt'}
        if nargout == 0
            fprintf(1, ' Demo scripts cannot use an option struct!\n');
        else
            opts = struct([]);
        end
        
    case { ... % Data scripts
            'morlab_data_desc_br', ...
            'morlab_data_desc_infunstab', ...
            'morlab_data_desc_pr', ...
            'morlab_data_so_stab', ...
            'morlab_data_std_br', ...
            'morlab_data_std_pr', ...
            'morlab_data_std_unstab'}
        if nargout == 0
            fprintf(1, ' Data scripts cannot use an option struct!\n');
        else
            opts = struct([]);
        end
        
    otherwise % Non-MORLAB functions.
        error('MORLAB:data', ...
            'The given function name is not implemented in MORLAB.');
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DEFAULT HELP LINE.                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Default help link for more details.
if nargout == 0
    fprintf(1, '\n');
    if hasHyperlink
        fprintf(1, [' For more details see <a href="matlab:help ' ...
            '%s">%s</a>.\n'], fname, fname);
    else
        fprintf(1, ' For more details see %s.', fname);
    end
    fprintf(1, '\n');
end
