function [] = ml_sigmaplot(sys, rom, wmin, wmax, n, bound, lspec)
%ML_SIGMAPLOT Sigma plot of error transfer functions in a given interval.
%
% SYNTAX:
%   ML_SIGMAPLOT(sys, [], wmin, wmax, n)
%   ML_SIGMAPLOT(sys, [], wmin, wmax, n, [], lspec)
%
%   ML_SIGMAPLOT(sys, rom, wmin, wmax, n)
%   ML_SIGMAPLOT(sys, rom, wmin, wmax, n, bound)
%   ML_SIGMAPLOT(sys, rom, wmin, wmax, n, bound, lspec)
%
% DESCRIPTION:
%   This function can be used to plot the error
%
%       sigma_max(G(j*w) - H(j*w))
%
%   in the interval j*[10^wmin, 10^wmax] at n points, where G is the
%   transfer function of the system sys and H is the transfer function
%   of the system rom. If no rom system is given, the plot shows the
%   magnitude of G.
%
% INPUT:
%   sys    - struct, containing a standard/descriptor/second-order system
%   rom    - struct, containing a standard/descriptor/second-order system,
%            if empty only the magnitude of sys is computed
%   wmin   - integer, lower exponent of the interval
%   wmax   - integer, higher exponent of the interval
%   n      - positive integer, number of points for computation
%   bound  - nonnegative scalar, absolute error bound, which shall be
%            plotted, otherwise is set empty
%   lspec  - character array, determining the color and line specification
%            of the plot
%
% OUTPUT:
%   ---
%
% See also ml_ct_dss_lti2struct, ml_ct_ss_lti2struct.

%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU Affero General Public License as published
% by the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Affero General Public License for more details.
%
% You should have received a copy of the GNU Affero General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Copyright (C) 2006-2018 Peter Benner, Steffen W. R. Werner
%


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHECK INPUTS.                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

narginchk(5, 7);

if nargin < 6
    bound = [];
end

if (nargin < 7) || isempty(lspec)
    lspec = 'b.';
end

% Check and assign input arguments.
if isfield(sys, 'M') && isfield(sys, 'E') && isfield(sys, 'K')
    % Second-order system.
    sys_type = 2;
    
    ml_assert_sosys(sys);
    
    if ~ml_field_set_to_value(sys, 'Cp')
        sys.Cp = zeros(size(sys.Cv));
    end
    
    if ~ml_field_set_to_value(sys, 'Cv')
        sys.Cv = zeros(size(sys.Cp));
    end
    
    if ml_field_set_to_value(sys, 'D')
        if ml_field_set_to_value(sys, 'Cp')
            assert(isequal(size(sys.D), ...
                [size(sys.Cp, 1), size(sys.Bu, 2)]), ...
                'MORLAB:data', ...
                'The matrix D must have the dimensions %d x %d!', ...
                size(sys.Cp, 1), size(sys.Bu, 2));
        else
            assert(isequal(size(sys.D), ...
                [size(sys.Cv, 1), size(sys.Bu, 2)]), ...
                'MORLAB:data', ...
                'The matrix D must have the dimensions %d x %d!', ...
                size(sys.Cv, 1), size(sys.Bu, 2));
        end
    else
        if ml_field_set_to_value(sys, 'Cp')
            sys.D = zeros(size(sys.Cp, 1), size(sys.Bu, 2));
        else
            sys.D = zeros(size(sys.Cv, 1), size(sys.Bu, 2));
        end
    end
else
    % First-order system.
    sys_type = 1;
    
    ml_assert_stdsys(sys);
    
    if ml_field_set_to_value(sys, 'D')
        assert( ...
            isequal(size(sys.D), [size(sys.C, 1), size(sys.B, 2)]), ...
            'MORLAB:data', ...
            'The matrix D must have the dimensions %d x %d!', ...
            size(sys.C, 1), size(sys.B, 2));
    else
        sys.D = zeros(size(sys.C, 1), size(sys.B, 2));
    end
    
    if ml_field_set_to_value(sys, 'E')
        assert(isequal(size(sys.E), size(sys.A)), ...
            'MORLAB:data', ...
            'The matrix E must have the same dimensions as A!');
    else
        sys.E = eye(size(sys.A));
    end
end

if ~isempty(rom)
    if isfield(rom, 'M') && isfield(rom, 'E') && isfield(rom, 'K')
        % Second-order system.
        rom_type = 2;
        
        ml_assert_sosys(rom);
        
        if ~ml_field_set_to_value(rom, 'Cp')
            rom.Cp = zeros(size(rom.Cv));
        end
        
        if ~ml_field_set_to_value(rom, 'Cv')
            rom.Cv = zeros(size(rom.Cp));
        end
        
        if ml_field_set_to_value(rom, 'D')
            if ml_field_set_to_value(rom, 'Cp')
                assert(isequal(size(rom.D), ...
                    [size(rom.Cp, 1), size(rom.Bu, 2)]), ...
                    'MORLAB:data', ...
                    'The matrix D must have the dimensions %d x %d!', ...
                    size(rom.Cp, 1), size(rom.Bu, 2));
            else
                assert(isequal(size(rom.D), ...
                    [size(rom.Cv, 1), size(rom.Bu, 2)]), ...
                    'MORLAB:data', ...
                    'The matrix D must have the dimensions %d x %d!', ...
                    size(rom.Cv, 1), size(rom.Bu, 2));
            end
        else
            if ml_field_set_to_value(rom, 'Cp')
                rom.D = zeros(size(rom.Cp, 1), size(rom.Bu, 2));
            else
                rom.D = zeros(size(rom.Cv, 1), size(rom.Bu, 2));
            end
        end
    else
        % First-order system.
        rom_type = 1;
        
        ml_assert_stdsys(rom);
        
        if ml_field_set_to_value(rom, 'D')
            assert( ...
                isequal(size(rom.D), [size(rom.C, 1), size(rom.B, 2)]), ...
                'MORLAB:data', ...
                'The matrix D must have the dimensions %d x %d!', ...
                size(rom.C, 1), size(rom.B, 2));
        else
            rom.D = zeros(size(rom.C, 1), size(rom.B, 2));
        end
        
        if ml_field_set_to_value(rom, 'E')
            assert(isequal(size(rom.E), size(rom.A)), ...
                'MORLAB:data', ...
                'The matrix E must have the same dimensions as A!');
        else
            rom.E = eye(size(rom.A));
        end
    end
else
    rom_type = 0;
end


ml_assert_integer(wmin, 'wmin');
ml_assert_integer(wmax, 'wmax');
ml_assert_posinteger(n, 'n');

if (nargin >= 6) && ~isempty(rom) && ~isempty(bound)
    ml_assert_nonnegscalar(bound, 'bound');
else
    bound = [];
end

assert(ischar(lspec), ...
    'MORLAB:data', ...
    'The argument lspec has to be a character array!');


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INITIALIZATION.                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

err  = zeros(1, n);
freq = logspace(wmin, wmax, n);


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COMPUTATION OF FUNCTION VALUES.                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for j = 1:1:n
    if sys_type == 1
        sysvalue = (sys.C / (1i * freq(j) * sys.E - sys.A)) ...
            * sys.B + sys.D;
    elseif sys_type == 2
        sysvalue = ((sys.Cp + 1i * freq(j) * sys.Cv) ...
            / (-freq(j)^2 * sys.M + 1i * freq(j) * sys.E + sys.K)) ...
            * sys.Bu + sys.D;
    end
    
    if rom_type == 1
        romvalue = (rom.C / (1i * freq(j) * rom.E - rom.A)) ...
            * rom.B + rom.D;
    elseif rom_type == 2
        romvalue = ((rom.Cp + 1i * freq(j) * rom.Cv) ...
            / (-freq(j)^2 * rom.M + 1i * freq(j) * rom.E + rom.K)) ...
            * rom.Bu + rom.D;
    elseif rom_type == 0
        romvalue = 0;
    end
    
    err(j) = max(svd(sysvalue - romvalue));
end

errmax = max(err);

if (nargin >= 6) && ~isempty(bound) && (errmax > bound)
    warning('MORLAB:sigmaplot', ...
        ['The maximum computed error exceds the given error bound!\n' ...
        'Error bound: %e | Max. comp. error: %e'], ...
        bound, errmax);
end


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CONSTRUCTION OF PLOT.                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (nargin >= 6) && ~isempty(bound)
    semilogx([freq(1), freq(n)], [bound, bound], 'r-', 'LineWidth', 1);
    hold on;
end

semilogx(freq, err, lspec, 'LineWidth', 1);
xlabel('Frequency \omega (rad/sec)');

if rom_type == 0
    ylabel('\sigma_{max}(G(j\omega))');
else
    ylabel('\sigma_{max}(G(j\omega) - H(j\omega))');
end

hold off;
